;------------------------------------------------------------------------------
; Disassembling a komentar Roman Brik - 1 Dec 2006 (Revizia 2)
;------------------------------------------------------------------------------
;
;               ***********************************************
;               * Ovladaci program disketovej jednotky PMD 32 *
;               ***********************************************
;
; PMD 32 je disketova jednotka s dvoma 5,25" mechanikami.
; Je riadena mikroprocesorom 8080A a vlastnym riadiacim programom o rozsahu asi
; 2 kB. Pamat RAM o velkosti 1kB sluzi ako operacna pamat mikroprocesora a
; vyrovnavacia pamat pre prenos dat medzi PMD 32 a PMD 85. Cast pamati RAM
; o velkosti asi 256 bytov je volna pre uzivatela.
; Zakladne diskove sluzby su vykonavane obvodom FDC 8272 v sucinnosti s obvodom
; DMA 8257.
; PMD 32 komunikuje s nadradenym pocitacom PMD 85 prostrednictvom paralelneho
; obojsmerneho kanala reprezentovaneho obvodom PIO 8255.
; Komunikacia medzi PMD 85 a PMD 32 je v rezime "MASTER - SLAVE", teda PMD 85
; posiela povely a PMD 32 na ne prislusne odpoveda.
; Zahajenie komunikacie je reprezentovane vymenou tzv. prezentacnych bytov.
; PMD 32 po zapnuti opakovane odosiela prezentacny byte a caka na prezentacny
; byte od PMD 85. Po prijati prezentacneho bytu od PMD 85 ocakava povel, ktory
; ma vykonat.
;
; PMD 32 moze pracovat v dvoch modoch:
;   SLOW - pri zapise kazdeho logickeho sektoru sa prevedie okamzite zapis na
;          disk (je aktivny po zapnuti alebo resete disketovej jednotky).
;   FAST - pri zapise logickeho sektoru sa tento ulozi do vyrovnavacej pamati
;          fyzickeho sektoru a ten sa ulozi na disk az v pripade, ze dalsi
;          zapisovany logicky sektor patri do ineho fyzickeho sektoru.
;
; PMD 32 ma implementovanu jednoduchu logiku oneskoreneho zastavovania mechanik,
; ak sa s nimi uz nepracuje. Zaroven sa pri zastavovani prislusnej mechaniky
; ulozi pripadny modifikovany fyzicky sektor. Tato logika umoznuje nechat pri
; opakovanom pristupe na disk roztocenu mechaniku a tym zrychlit citanie a zapis
; na disk.
;
; PMD 32 poskytuje tieto povely:
;
; povel U (0227) - zapis do pamati PMD 32
; povel C (0239) - citanie z pamati PMD 32
; povel J (0258) - vykonanie podprogramu v pamati PMD 32
; povel * (0263) - prepnutie PMD 32 do FAST rezimu
; povel @ (026C) - prepnutie PMD 32 do SLOW rezimu
; povel B (02A5) - vyziadanie BOOT sektora
; povel Q (02B7) - precitanie logickeho sektora
; povel R (02D6) - precitanie logickeho sektora
; povel S (02E5) - zapis fyzickeho sektora
; povel T (0323) - zapis logickeho sektora
; povel W (0345) - zapis logickeho sektora
; povel I (03BD) - vyber mechaniky a presun hlavy na stopu 0
; povel F (03DB) - format stopy
;
; Format jednotlivych povelov je popisany priamo v kode. V popise sa vyskytuju
; tieto skratky:
;   ACK - potvrdenie povelu disketovou jednotkou. Tento byte ma hodnotu 33h
;   NAK - NEpotvrdenie povelu disketovou jednotkou alebo chyba pri komunikacii
;         s PMD 85 alebo FDC. Tento byte ma hodnotu 99h
;   ERR - je kod oznacujuci chybovy kod vysledku pozadovaneho povelu
;   CRC - kontrolny sucet bloku dat - XOR vsetkych bytov bloku mimo ACK a ERR
; Prezentacny byte ma hodnotu AAh.

;------------------------------------------------------------------------------
; Rozdelenie adries periferii v PMD 32
; -------------------------------------
; * FDC 8272 *
; 00 - R/O - status register
; 01 - R/W - datovy register
;
; * PIO 8255 *
; 20 - R/W - port A
; 21 - R/W - port B - nevyuzite
; 22 - R/W - port C
; 23 - W/O - riadiaci port
;
; * DMA 8257 *
; 40 - R/W - register adresy - DMA kanal 0 - iba zapis
; 41 - R/W - register pocitadla a operacie - DMA kanal 0 - iba zapis
; 42 - R/W - register adresy - DMA kanal 1 - nevyuzite
; 43 - R/W - register pocitadla a operacie - DMA kanal 1 - nevyuzite
; 44 - R/W - register adresy - DMA kanal 2 - nevyuzite
; 45 - R/W - register pocitadla a operacie - DMA kanal 2 - nevyuzite
; 46 - R/W - register adresy - DMA kanal 3 - nevyuzite
; 47 - R/W - register pocitadla a operacie - DMA kanal 3 - nevyuzite
; 48 - R/W - status/mode register - iba zapis
;
; * register vyberu mechanik a motorov *
; E0 - W/O -  7   6   5   4   3   2   1   0
;            DS1 DS0 MO1 MO0 ENA  0   0   0


;------------------------------------------------------------------------------
; Start
;------------------------------------------------------------------------------
0000 LXI  SP,1C00 ; zasobnik na koniec pamati
0003 JMP  00CB    ; skok na inicializaciu


;------------------------------------------------------------------------------
0006 DB   8B,1E   ; seno
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; RST 1 - Precita byte zo vstupu (z PMD 85)
; vstup: B = priebezne CRC
; vystup: A = precitany byte, B = modifikovane CRC
;         skok do ERROR rutiny, ak dojde k timeoutu
; meni: A, B
;------------------------------------------------------------------------------
0008 CALL 0051    ; pockaj na byte na vstupe
000B IN   20      ; precitaj byte
000D PUSH PSW     ; odpamataj AF
000E XRA  B       ; modifikuj CRC
000F MOV  B,A     ; CRC naspat do B
0010 POP  PSW     ; obnov AF
0011 RET


;------------------------------------------------------------------------------
0012 DB   1F,9A,A1,9B,16,16   ; seno
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; RST 3 - Zapise byte na vystup (do PMD 85)
; vstup: B = priebezne CRC
; vystup: A = B = modifikovane CRC
;         skok do ERROR rutiny, ak dojde k timeoutu
; meni: A, B
;------------------------------------------------------------------------------
0018 CALL 0051    ; pockaj kym sa uvolni vystup
001B OUT  20      ; posli byte
001D XRA  B       ; modifikuj CRC
001E MOV  B,A     ; CRC naspat do B
001F RET


;------------------------------------------------------------------------------
; RST 4 - Povoli prerusenie pre vstup na porte A
; vstup: -
; vystup: povolene prerusenie pre vstup bytu
; meni: A
;------------------------------------------------------------------------------
0020 IN   20      ; preventivne vyprazdni vstupny latch portu A
0022 MVI  A,09    ; 1->C4 povolenie prerusovacieho signalu INTRA pre vstup A
0024 JMP  003B    ; skok dopredu


;------------------------------------------------------------------------------
0027 DB   89      ; seno
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; RST 5 - Povoli prerusenie pre vystup na porte A
; vstup: -
; vystup: povolene prerusenie pre vystup bytu
; meni: A
;------------------------------------------------------------------------------
0028 IN   20      ; preventivne vyprazdni vstupny latch portu A
002A MVI  A,08    ; 0->C4 zakazanie prerusovacieho signalu INTRA pre vstup A
002C JMP  0042    ; skok dopredu


;------------------------------------------------------------------------------
002F DB   43      ; seno
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; RST 6 - Precita zo vstupu (z PMD 85) byte reprezentujuci CRC a ak je CRC
;         v poriadku odosle potvrdenie ACK.
;         V opacnom pripade skoci do ERROR rutiny.
; vstup: B = priebezne CRC
; vystup: odoslane ACK, ak bolo CRC v poriadku a nedoslo k timeoutu
;         odoslane NAK v ERROR rutine
; meni: A, B
;------------------------------------------------------------------------------
0030 RST  1       ; precitaj byte
0031 MOV  A,B     ; CRC do A
0032 ORA  A       ; CRC musi byt nulove
0033 JMP  0049    ; skok dopredu


;------------------------------------------------------------------------------
0036 DB   9A,B4   ; seno
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; Rutina prerusenia.
; Podla potreby je cielova adresa skoku modifikovana v RAM na adrese 1804.
;------------------------------------------------------------------------------
0038 JMP  1803    ; skok do RAM, kde je instrukcia skoku do prislusnej rutiny


;------------------------------------------------------------------------------
; Pokracovanie rutiny RST 4
; Povoli prerusenie pre vstup na porte A
;------------------------------------------------------------------------------
003B OUT  23
003D MVI  A,0C    ; 0->C6 zakazanie prerusovacieho signalu INTRA pre vystup A
003F OUT  23
0041 RET


;------------------------------------------------------------------------------
; Pokracovanie rutiny RST 5
; Povoli prerusenie pre vystup na porte A
;------------------------------------------------------------------------------
0042 OUT  23
0044 MVI  A,0D    ; 1->C6 povolenie prerusovacieho signalu INTRA pre vystup A
0046 OUT  23
0048 RET


;------------------------------------------------------------------------------
; Pokracovanie rutiny RST 6
; Precita zo vstupu (z PMD 85) byte reprezentujuci CRC
;------------------------------------------------------------------------------
0049 JNZ  00C0    ; ak CRC nesedi skoc do ERROR rutiny
004C RST  5       ; povol prerusenie pre vystup bytu
004D MVI  A,33    ; ACK do A
004F RST  3       ; posli ACK
0050 RET


;------------------------------------------------------------------------------
; Zisti, ci je nastavene INTRA.
; INTRA=1, ak je platny byte na vstupe, alebo bol precitany byte z vystupu.
; vstup: -
; vystup: skok do ERROR rutiny, ak dojde k timeoutu
;         inak normalny navrat
; meni: -
;------------------------------------------------------------------------------
0051 PUSH PSW     ; odpamataj AF a BC
0052 PUSH B
0053 MVI  C,00    ; timeout asi 5 ms
0055 DCR  C       ; zniz pocitadlo
0056 JZ   00C0    ; ak je nulove, skoc do ERROR rutiny
0059 IN   22      ; zisti stav INTRA
005B ANI  08
005D JZ   0055    ; ak je INTRA=0, vrat sa do slucky
0060 POP  B       ; INTRA=1, obnov BC a AF
0061 POP  PSW
0062 RET


;------------------------------------------------------------------------------
; Rutina zabezpecuje automaticke "oneskorene" vypnutie motorov mechanik po
; ukonceni povelu. To umoznuje nechat tocit mechaniku po celu pracu s diskom.
; Pred zastavenim mechaniky sa najprv ulozi aktualny sektor v buffri, ak bol
; medzi tym modifikovany.
; vstup: -
; vystup: skok do ERROR rutiny, ak dojde ku nejakej chybe
;         inak normalny navrat
; meni: -
;------------------------------------------------------------------------------
0063 LHLD 1831    ; timer vypnutia mechaniky A
0066 DCX  H       ; zniz pocitadlo
0067 SHLD 1831    ; uloz naspat
006A MOV  A,H
006B ORA  A
006C JNZ  0091    ; ak nie je vynulovane, skoc dopredu
006F CALL 051A    ; urob prepocet cisla sektora - B = cislo mechaniky
0072 DCR  B       ; ak je aktualnou mechanikou B,
0073 JNZ  007D    ; skoc dopredu
0076 CALL 0382    ; uloz aktualny fyzicky sektor
0079 XRA  A       ; flag - aktualny fyzicky sektor je ulozeny
007A STA  183D
007D LDA  1837    ; vyber stav portu mechanik
0080 ANI  A0      ; ponechaj iba bity DS1 a MO1
0082 STA  1837    ; a uloz naspat
0085 JNZ  008D    ; ak sa uz netoci ziadna z mechanik,
0088 LXI  H,183C
008B MVI  M,FF    ; zneplatni naposledy pouzity logicky sektor a mechaniku
008D ORI  08      ; pridaj ENA bit
008F OUT  E0      ; a zapis na port mechanik
0091 LHLD 1833    ; timer vypnutia mechaniky B
0094 DCX  H       ; zniz pocitadlo
0095 SHLD 1833    ; uloz naspat
0098 MOV  A,H
0099 ORA  A
009A JNZ  00BF    ; ak nie je vynulovane, skoc dopredu
009D CALL 051A    ; urob prepocet cisla sektora - B = cislo mechaniky
00A0 DCR  B       ; ak je aktualnou mechanikou A,
00A1 JZ   00AB    ; skoc dopredu
00A4 CALL 0382    ; uloz aktualny fyzicky sektor
00A7 XRA  A       ; flag - aktualny fyzicky sektor je ulozeny
00A8 STA  183D
00AB LDA  1837    ; vyber stav portu mechanik
00AE ANI  50      ; ponechaj iba bity DS0 a MO0
00B0 STA  1837    ; a uloz naspat
00B3 JNZ  00BB    ; ak sa uz netoci ziadna z mechanik,
00B6 LXI  H,183C
00B9 MVI  M,FF    ; zneplatni naposledy pouzity logicky sektor a mechaniku
00BB ORI  08      ; pridaj ENA bit
00BD OUT  E0      ; a zapis na port mechanik
00BF RET


;------------------------------------------------------------------------------
; ERROR - Rutina odosle byte NAK na vystup, nastavi SP na pociatocnu hodnotu
;         a skoci do hlavnej slucky.
; vstup: -
; vystup: -
; meni: A, SP
;------------------------------------------------------------------------------
00C0 LXI  SP,1C00 ; zasobnik na koniec pamati
00C3 RST  5       ; povol prerusenie pre vystup bytu
00C4 MVI  A,99    ; NAK do A
00C6 OUT  20      ; odosli
00C8 JMP  013F    ; skoc do hlavnej slucky


;------------------------------------------------------------------------------
; Inicializacia.
; Po Resete sa prevedie inicializacia PIO, FDC, inicializacia premennych v RAM
; a hlavy oboch mechanik sa poslu domov (na stopu 0).
;------------------------------------------------------------------------------
00CB DI           ; zakaz prerusenie procesora
00CC MVI  A,30    ; zapni motory oboch mechanik (snad)
00CE OUT  E0
00D0 MVI  A,C0    ; inicializacia PIO 8255
00D2 OUT  23      ; SK A: MOD 2, A-IN/OUT,  SK B: MOD 0, B-OUT, CL-OUT
00D4 LXI  H,065F  ; presun casti kodu a dat do RAM
00D7 LXI  D,1800  ; od adresy 1800
00DA MVI  B,3F    ; celkom 63 bytov
00DC MOV  A,M     ; vyber byte z ROM
00DD STAX D       ; uloz do RAM
00DE INX  H       ; posun ukazovatele
00DF INX  D
00E0 DCR  B       ; zniz pocitadlo
00E1 JNZ  00DC    ; opakuj B-krat
00E4 MVI  A,38    ; zapni motory oboch mechanik
00E6 OUT  E0
00E8 LXI  H,0647  ; adresa povelu pre nastavenie specifikacie FDC
00EB CALL 01BB    ; zapis povel do FDC
00EE XRA  A       ; cislo logickeho sektora = 0, mechanika A - v 7. bite
00EF STA  183A
00F2 MVI  A,78    ; vyber mechaniku A
00F4 OUT  E0
00F6 LXI  D,064B  ; adresa povelu pre nastavenie hlavy mechaniky A na stopu 0
00F9 LXI  H,0200  ; adresa prerusovacej ruitny pre prijatie vysledku z FDC
00FC CALL 01F9    ; zapis do FDC
00FF XRA  A       ; hlava mechaniky A je na stope 0
0100 STA  1835
0103 MVI  A,B8    ; vyber mechaniku B
0105 OUT  E0
0107 MVI  A,80    ; cislo logickeho sektora = 0, mechanika B - v 7. bite
0109 STA  183A
010C LXI  D,064E  ; adresa povelu pre nastavenie hlavy mechaniky B na stopu 0
010F LXI  H,0200  ; adresa prerusovacej ruitny pre prijatie vysledku z FDC
0112 CALL 01F9    ; zapis do FDC
0115 XRA  A       ; hlava mechaniky B je na stope 0
0116 STA  1836
0119 MVI  A,38    ; zrus vyber oboch mechanik, motory nechaj zapnute
011B OUT  E0
011D JMP  0137    ; skoc do slucky odosielania prezentacneho bytu


;------------------------------------------------------------------------------
; Tu sa skace po vykonani povelu DJ.
; Preveruje sa, je na vstupe dalsi byte, a ak ano, spracuje sa.
;------------------------------------------------------------------------------
0120 LXI  SP,1C00 ; zasobnik na koniec RAM
0123 RST  4       ; povol prerusenie PIO pre vstup
0124 CALL 0063    ; obsluz automaticke zastavenie mechanik
0127 LXI  D,03E8  ; timeout asi 25 ms
012A IN   22      ; poslalo PMD 85 byte (dalsi povel) ?
012C ANI  08
012E JNZ  0164    ; ak ano, skoc dopredu
0131 DCX  D       ; zniz pocitadlo timeoutu
0132 MOV  A,D
0133 ORA  E
0134 JNZ  012A    ; ak nie je nulove, cakaj


;------------------------------------------------------------------------------
; Hlavna slucka.
; Zabezpecuje "naviazanie spojenia" prostrednictvom vymeny prezentacnych bytov,
; prijatie povelu od PMD 85 a skok na vykonanie prislusneho povelu.
;------------------------------------------------------------------------------
0137 LXI  SP,1C00 ; zasobnik na koniec RAM
013A RST  5       ; povol prerusenie PIO na vystup
013B MVI  A,AA    ; prezentacny byte
013D OUT  20      ; posli na vystup
013F CALL 0063    ; obsluz automaticke zastavenie mechanik
0142 MVI  C,96    ; timeout asi 3 ms
0144 DCR  C
0145 JZ   013B    ; PMD 85 si neprevzalo prezentacny byte, skoc odoslat znovu
0148 IN   22      ; prijalo PMD 85 prezentacny byte ?
014A ANI  08
014C JZ   0144    ; ak nie, cakaj
014F RST  4       ; povol prerusenie PIO pre vstup
0150 MVI  B,00    ; timeout asi 5 ms
0152 DCR  B
0153 JZ   0137    ; PMD 85 neposlalo prezentacny byte, skoc naspat do slucky
0156 IN   22      ; poslalo PMD 85 byte ?
0158 ANI  08
015A JZ   0152    ; ak nie, cakaj
015D IN   20      ; precitaj byte
015F CPI  AA      ; je to prezentacny byte ?
0161 JNZ  0137    ; ak nie, vrat sa do slucky
0164 RST  1       ; skus precitat dalsi byte, mal by to byt povel
0165 MOV  B,A     ; uloz prijaty byte do B a testuj ci je to znamy povel
0166 CPI  57      ; povel W
0168 JZ   0345
016B CPI  52      ; povel R
016D JZ   02D6
0170 CPI  55      ; povel U
0172 JZ   0227
0175 CPI  43      ; povel C
0177 JZ   0239
017A CPI  49      ; povel I
017C JZ   03BD
017F CPI  4A      ; povel J
0181 JZ   0258
0184 CPI  42      ; povel B
0186 JZ   02A5
0189 CPI  51      ; povel Q
018B JZ   02B7
018E CPI  54      ; povel T
0190 JZ   0323
0193 CPI  53      ; povel S
0195 JZ   02E5
0198 CPI  46      ; povel F
019A JZ   03DB
019D CPI  2A      ; povel *
019F JZ   0263
01A2 CPI  40      ; povel @
01A4 JZ   026C
01A7 MOV  B,A     ; uloz povel do B
01A8 LDA  0800    ; mame rozsirenu ROM od adresy 0800 ?
01AB CPI  C3
01AD MOV  A,B     ; povel naspat do A
01AE JZ   0800    ; ak mame rozsirenu ROM, skoc overit ci je povel znamy
01B1 CALL 1800    ; zavolaj podprogram v RAM - je tam skok na RET
01B4 JMP  0137    ; vrat sa do slucky odosielania prezentacneho bytu


;------------------------------------------------------------------------------
; Zapis do FDC s nastavenim adresy prerusovacej rutiny.
; vstup: DE = adresa zdrojoveho buffru. 1. byte buffra obsahuje celkovy pocet
;             zapisovanych bytov
;        HL = adresa rutiny prerusenia
; vystup: -
;         RESET, ak sa FDC dostane do neocakavaneho stavu
; meni: AF, B, DE, HL
;------------------------------------------------------------------------------
01B7 SHLD 1804
01BA XCHG


;------------------------------------------------------------------------------
; Zapis do FDC.
; vstup: HL = adresa zdrojoveho buffru. 1. byte buffra obsahuje celkovy pocet
;             zapisovanych bytov
; vystup: -
;         RESET, ak sa FDC dostane do neocakavaneho stavu
; meni: AF, B, HL
;------------------------------------------------------------------------------
01BB DI           ; zakaz prerusenie
01BC MOV  B,M     ; do B pocet bytov na zapisanie do FDC
01BD IN   00      ; je FDC pripravene ? (7 bit status registra)
01BF ADD  A
01C0 JNC  01BD    ; ak nie, cakaj
01C3 INX  H       ; posun ukazovatela na dalsi byte
01C4 MOV  A,M     ; zapisovany byte do A
01C5 OUT  01      ; zapis do FDC
01C7 ORA  A       ; vynuluj CY
01C8 DCR  B       ; zniz pocitadlo zapisovanych bytov
01C9 RZ           ; vrat sa, ak boli zapisane vsetky
01CA XTHL         ; nutne zdrzanie pred citanim status registra po zapise
01CB XTHL
01CC IN   00      ; je FDC pripravene ? (7 bit status registra)
01CE ADD  A
01CF JNC  01CC    ; ak nie, cakaj
01D2 IN   00      ; zisti smer prenosu (6 bit status registra)
01D4 ANI  40
01D6 JZ   01C3    ; ak je to FDC <- CPU, skoc zapisat dalsi byte
01D9 RST  0       ; chyba - FDC je v neocakavanom stave - preved RESET


;------------------------------------------------------------------------------
; Citanie z FDC.
; vstup: HL = adresa cieloveho buffru. 1. byte buffra obsahuje celkovy pocet
;             citanych bytov
; vystup: -
;         RESET, ak sa FDC dostane do neocakavaneho stavu
; meni: AF, B, HL
;------------------------------------------------------------------------------
01DA DI           ; zakaz prerusenie
01DB MOV  B,M     ; do B pocet citanych bytov z FDC
01DC IN   00      ; je FDC pripravene ? (7 bit status registra)
01DE ADD  A
01DF JNC  01DC    ; ak nie, cakaj
01E2 INX  H       ; posun ukazovatel
01E3 IN   01      ; precitaj byte z FDCD
01E5 MOV  M,A     ; uloz do pamati
01E6 ORA  A       ; vynuluj CY
01E7 DCR  B       ; zniz pocitadlo citanych bytov
01E8 RZ           ; vrat sa, ak boli precitane vsetky
01E9 XTHL         ; nutne zdrzanie pred citanim status registra po citani dat
01EA XTHL
01EB IN   00      ; je FDC pripravene ? (7 bit status registra)
01ED ADD  A
01EE JNC  01EB    ; ak nie, cakaj
01F1 IN   00      ; zisti smer prenosu (6 bit status registra)
01F3 ANI  40
01F5 JNZ  01E2    ; ak je to CPU <- FDC, skoc precitat dalsi byte
01F8 RST  0       ; chyba - FDC je v neocakavanom stave - preved RESET


;------------------------------------------------------------------------------
; Zapis do FDC s nastavenim adresy prerusovacej rutiny.
; vstup: DE = adresa zdrojoveho buffru. 1. byte buffra obsahuje celkovy pocet
;             zapisovanych bytov
;        HL = adresa rutiny prerusenia
; vystup: -
;         RESET, ak sa FDC dostane do neocakavaneho stavu
; meni: AF, B, DE, HL
;------------------------------------------------------------------------------
01F9 CALL 01B7    ; nastav adresu prerusenia a posli povel do FDC
01FC EI           ; povol prerusenie
01FD HLT          ; a cakaj na vysledok
; V oboch prerusovacich rutinach sa navratova adresa "zahadzuje", takze realny
; navrat za HLT sa nikdy neuskutocni.


;------------------------------------------------------------------------------
; 1. Rutina prerusenia CPU.
; Tato rutina prerusenia prevedie len jednoduchy navrat do volajucj rutiny.
;------------------------------------------------------------------------------
01FE POP  H       ; zahod "navratovu adresu" prerusenia
01FF RET          ; a vrat sa do volajucej rutiny


;------------------------------------------------------------------------------
; 2. Rutina prerusenia CPU.
; Tato rutina prerusenia posle do FDC povel pre ziskanie vysledku operacie
; a vrati sa do volajucej rutiny.
;------------------------------------------------------------------------------
0200 POP  H       ; zahod "navratovu adresu" prerusenia
0201 LXI  D,0651  ; adresa povelu pre pre zistenie dovodu prerusenia FDC
0204 LXI  H,01FE  ; z prerusenia sa len vrat
0207 CALL 01B7    ; posli povel do FDC
020A LXI  H,182C  ; adresa buffra pre precitanie ST0 a cylindra z FDC
020D CALL 01DA    ; precitaj z FDC
0210 CALL 051A    ; urob prepocet cisla sektora - v B je cislo mechaniky
0213 MVI  A,02    ; maska na mechaniku B
0215 ANA  B
0216 MOV  B,A     ; uloz do B
0217 LDA  182D    ; ST0 do A
021A ANI  03      ; odmaskuj US1 a US2 - mechanika
021C CMP  B       ; porovnaj
021D RZ           ; su zhodne, vrat sa
021E LXI  H,0200  ; nebol to "nas vysledok"
0221 SHLD 1804    ; nastav opat adresu tohoto prerusenia
0224 JMP  01FC    ; a skoc cakat na vysledok


;------------------------------------------------------------------------------
; povel U - zapis do pamati PMD 32
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "U"                     | ---> |                         |
;        | vyssi byte adresy       | ---> |                         |
;        | nizsi byte adresy       | ---> |                         |
;        | vyssi byte poctu        | ---> |                         |
;        | nizsi byte poctu        | ---> |                         |
;        | 'pocet' bytov dat       | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba
; meni: vsetky
;------------------------------------------------------------------------------
0227 CALL 024B    ; prijmi do HL cielovu adresu a do DE pocet
022A RST  1       ; prijmi byte
022B MOV  M,A     ; uloz do pamati
022C INX  H       ; posun ukazatel
022D DCX  D       ; zniz pocitadlo
022E MOV  A,D
022F ORA  E
0230 JNZ  022A    ; ak nie je nulove, opakuj
0233 RST  6       ; prijmi CRC a odosli ACK
0234 XRA  A       ; kod chyby 00 - OK
0235 RST  3       ; posli vysledok operacie
0236 JMP  0120    ; navrat do povelovej slucky


;------------------------------------------------------------------------------
; povel C - citanie z pamati PMD 32
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "C"                     | ---> |                         |
;        | vyssi byte adresy       | ---> |                         |
;        | nizsi byte adresy       | ---> |                         |
;        | vyssi byte poctu        | ---> |                         |
;        | nizsi byte poctu        | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | 'pocet' bytov dat       |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               >00 - chybne CRC
; meni: vsetky
;------------------------------------------------------------------------------
0239 CALL 024B    ; prijmi do HL zdrojovu adresu a do DE pocet
023C RST  6       ; prijmi CRC a odosli ACK
023D MOV  A,M     ; byte z pamati
023E RST  3       ; odosli
023F INX  H       ; posun ukazatel
0240 DCX  D       ; zniz pocitadlo
0241 MOV  A,D
0242 ORA  E
0243 JNZ  023D    ; ak nie je nulove, opakuj
0246 MOV  A,B     ; CRC
0247 RST  3       ; odosli CRC
0248 JMP  0120    ; navrat do povelovej slucky


;------------------------------------------------------------------------------
; Prijme styri byty a ulozi do HL a DE - zvycajne to predstavuje adresu a pocet.
; vstup: B = priebezne CRC
; vystup: A = E, HL = prva 2 bytova hodnota, DE = druha 2 bytova hodnota
;         skok do ERROR rutiny, ak dojde k timeoutu
; meni: A, B, HL
;------------------------------------------------------------------------------
024B CALL 0253    ; prijmi adresu do HL
024E RST  1       ; prijmi byte
024F MOV  D,A     ; uloz do D
0250 RST  1       ; prijmi byte
0251 MOV  E,A     ; uloz do E
0252 RET


;------------------------------------------------------------------------------
; Prijme dva byty a ulozi do HL - zvycajne to predstavuje adresu.
; vstup: B = priebezne CRC
; vystup: A = L, HL = 2 bytova hodnota
;         skok do ERROR rutiny, ak dojde k timeoutu
; meni: A, B, HL
;------------------------------------------------------------------------------
0253 RST  1       ; prijmi byte
0254 MOV  H,A     ; uloz do H
0255 RST  1       ; prijmi byte
0256 MOV  L,A     ; uloz do L
0257 RET


;------------------------------------------------------------------------------
; povel J - vykonanie podprogramu v pamati PMD 32
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "J"                     | ---> |                         |
;        | vyssi byte adresy       | ---> |                         |
;        | nizsi byte adresy       | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba
; meni: vsetky
;------------------------------------------------------------------------------
0258 LXI  H,0120  ; navratova adresa z podprogramu do slucky
025B PUSH H       ; na zasobnik
025C CALL 0253    ; do HL prijmi adresu podprogramu
025F RST  6       ; prijmi CRC a odosli ACK
0260 XRA  A       ; kod chyby 00 - OK
0261 RST  3       ; posli vysledok operacie
0262 PCHL         ; skok do podprogramu


;------------------------------------------------------------------------------
; povel * - prepnutie PMD 32 do FAST rezimu
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "*"                     | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba
; meni: vsetky
;------------------------------------------------------------------------------
0263 RST  6       ; prijmi CRC a odosli ACK
0264 MVI  A,FF    ; nastav FAST rezim
0266 STA  183E
0269 JMP  0234    ; vrat sa do slucky cez odoslanie OK chyboveho kodu


;------------------------------------------------------------------------------
; povel @ - prepnutie PMD 32 do SLOW rezimu
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "@"                     | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
026C RST  6       ; prijmi CRC a odosli ACK
026D CALL 0382    ; pred prechodom do SLOW rezimu uloz aktualny fyzicky sektor
0270 XRA  A       ; nastav SLOW rezim
0271 STA  183E
0274 JMP  0234    ; vrat sa do slucky cez odoslanie OK chyboveho kodu


;------------------------------------------------------------------------------
; Povel pre reset disketovej jednotky - nevyuzity
; Je ho mozne nahradit povelom "J"
;------------------------------------------------------------------------------
0277 RST  6       ; prijmi CRC a posli ACK
0278 RST  0       ; RESET


;------------------------------------------------------------------------------
; Rutina pre prijem cisla sektora, mechaniky a stopy
; vstup: B = priebezne CRC
; vystup: A = cislo stopy, B = modifikovane CRC
;         skok do ERROR rutiny, ak dojde k timeoutu
; meni: A, B
;------------------------------------------------------------------------------
0279 RST  1       ; prijmi byte
027A STA  183A    ; uloz cislo sektora + cislo mechaniky v 7. bite
027D RST  1       ; prijmi byte
027E STA  183B    ; uloz cislo stopy
0281 RET


;------------------------------------------------------------------------------
; Vypocita adresu buffra, kde sa nachadza pozadovany logicky sektor.
; vstup: B = priebezne CRC
; vystup: HL = adresa buffra pozadovaneho sektora
;         A = 0 - chybovy kod OK
;         B = 0 - pociatocne CRC
;         C = 128 - velkost logickeho sektora
; meni: vsetky
;------------------------------------------------------------------------------
0282 CALL 051A    ; urob prepocet cisla sektora
0285 MVI  H,00    ; HL = cislo logickeho sektora v ramci fyzickeho sektora
0287 DAD  H       ; x 2
0288 DAD  H       ; x 4
0289 DAD  H       ; x 8
028A DAD  H       ; x 16
028B DAD  H       ; x 32
028C DAD  H       ; x 64
028D DAD  H       ; x 128
028E LXI  D,18BF  ; adresa buffra
0291 DAD  D       ; pripocitaj offset - HL = adresa buffra pozadovaneho sektora
0292 MVI  A,00    ; 00 - kod chyby OK
0294 MVI  B,00    ; CRC
0296 MVI  C,80    ; velkost logickeho sektora
0298 RET


;------------------------------------------------------------------------------
; Rutina prijme cislo sektora, cislo stopy, CRC a odosle ACK.
; Potom precita z disku prislusny sektor a odosle chybovy kod 0 - OK, ak
; medzi tym nedoslo ku nejakej chybe.
; vstup: B = priebezne CRC
; vystup: HL = adresa bufru, kde je pripraveny prislusny sektor
;         A = 0 - kod chyby
;         B = 0 - CRC
;         C = 128 - velkost sektora
;         skok do ERROR rutiny, ak dojde ku nejakej chybe
; meni: vsetky
;------------------------------------------------------------------------------
0299 CALL 0279    ; prijmi cislo sektora a stopy
029C RST  6       ; prijmi CRC a odosli ACK
029D CALL 0585    ; priprav pozadovany logicky sektor do pamati
02A0 CALL 0282    ; vypocitaj adresu buffra, kde sa nachadza sektor
02A3 RST  3       ; odosli chybovy kod 0 - OK
02A4 RET


;------------------------------------------------------------------------------
; povel B - vyziadanie BOOT sektora
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "B"                     | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |                         | <--- | 128 bytov dat           |
;        |                         | <--- | CRC                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
02A5 XRA  A       ; BOOT musi byt na disku mechaniky A
02A6 STA  183A    ; v 0. logickom sektore
02A9 STA  183B    ; v 0. stope
02AC RST  6       ; prijmi CRC a odosli ACK
02AD CALL 0585    ; priprav pozadovany logicky sektor do pamati
02B0 CALL 0282    ; vypocitaj adresu buffra, kde sa nachadza sektor
02B3 RST  3       ; odosli chybovy kod 0 - OK
02B4 JMP  02BA    ; posli data BOOT sektora


;------------------------------------------------------------------------------
; povel Q - precitanie logickeho sektora z disku
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "Q"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |                         | <--- | 128 bytov dat           |
;        |                         | <--- | CRC                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
02B7 CALL 0299    ; prijmi cislo sektora a stopy, CRC, odosli ACK a 00 - OK
02BA MVI  D,00    ; timeout asi 5 ms
02BC DCR  D
02BD JZ   00C0    ; timeout - skok do ERROR rutiny
02C0 IN   22      ; je volny vystup ?
02C2 ANI  08
02C4 JZ   02BC    ; ak nie, cakaj
02C7 MOV  A,M     ; byte z pamati
02C8 OUT  20      ; posli
02CA XRA  B       ; modifikuj CRC
02CB MOV  B,A     ; uloz naspat do B
02CC INX  H       ; posun ukazovatel
02CD DCR  C       ; zniz pocitadlo
02CE JNZ  02BA    ; opakuj C-krat
02D1 MOV  A,B     ; CRC do A
02D2 RST  3       ; odosli CRC
02D3 JMP  0120    ; navrat do povelovej slucky


;------------------------------------------------------------------------------
; povel R - precitanie logickeho sektora z disku
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "R"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |                         | <--- | 128 bytov dat           |
;        |                         | <--- | CRC                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
02D6 CALL 0299    ; prijmi cislo sektora a stopy, CRC, odosli ACK a 00 - OK
02D9 MOV  A,M     ; byte z pamati
02DA RST  3       ; odosli
02DB INX  H       ; posun ukazovatel
02DC DCR  C       ; zniz pocitadlo
02DD JNZ  02D9    ; opakuj C-krat
02E0 MOV  A,B     ; CRC do A
02E1 RST  3       ; odosli CRC
02E2 JMP  0120    ; navrat do povelovej slucky


;------------------------------------------------------------------------------
; povel S - zapis fyzickeho sektora
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "S"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | 513 bytov dat           | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
02E5 CALL 0279    ; prijmi cislo sektora a stopy
02E8 LXI  D,0300  ; pocitadlo bytov - 513
02EB LXI  H,18BF  ; adresa buffra
02EE MVI  C,00    ; timeout 5 ms
02F0 DCR  C
02F1 JZ   00C0    ; timeout - skok do ERROR rutiny
02F4 IN   22      ; je byte na vstupe ?
02F6 ANI  08
02F8 JZ   02F0    ; ak nie, cakaj
02FB IN   20      ; precitaj byte
02FD MOV  M,A     ; uloz do pamati
02FE XRA  B       ; modifikuj CRC
02FF MOV  B,A     ; uloz naspat do B
0300 INX  H       ; posun ukazovatel
0301 DCX  D       ; zniz pocitadlo
0302 MOV  A,D
0303 ANA  A
0304 JNZ  02EE    ; opakuj 513-krat
0307 RST  6       ; prijmi CRC a odosli ACK
0308 CALL 039E    ; zisti, ci je disk chraneny proti zapisu
030B JNZ  00C0    ; ak ano, skoc do ERROR rutiny
030E LDA  183D    ; je aktualny fyzicky sektor ulozeny ?
0311 ORA  A
0312 JNZ  00C0    ; nie, je to chyba, skok do ERROR rutiny
0315 CALL 061F    ; rozbehni mechaniku a nastav pozadovanu stopu
0318 CALL 05E5    ; odzalohuj premenne
031B MVI  A,FF    ; flag - aktualny fyzicky sektor v buffri je modifikovany
031D STA  183D
0320 JMP  0378    ; skoc ulozit sektor a poslat prislusny chybovy kod


;------------------------------------------------------------------------------
; povel T - zapis logickeho sektora na disk
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "T"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | 128 bytov dat           | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
0323 CALL 0279    ; prijmi cislo sektora a stopy
0326 MVI  C,80    ; logicky sektor ma 128 bytov
0328 LXI  H,183F  ; adresa cieloveho buffra
032B MVI  D,00    ; timeout 5 ms
032D DCR  D       ; zniz pocitadlo
032E JZ   00C0    ; ak je nulove, skoc do ERROR rutiny
0331 IN   22      ; je na vstupe byte
0333 ANI  08
0335 JZ   032D    ; ak nie, cakaj
0338 IN   20      ; prijmi byte
033A MOV  M,A     ; uloz do pamati
033B XRA  B       ; modifikuj CRC
033C MOV  B,A     ; CRC naspat do B
033D INX  H       ; posun ukazovatel
033E DCR  C       ; zniz pocitadlo prijimanych bytov
033F JNZ  032B    ; opakuj 128-krat
0342 JMP  0354    ; skoc dalej


;------------------------------------------------------------------------------
; povel W - zapis logickeho sektora na disk
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "W"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | 128 bytov dat           | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
0345 CALL 0279    ; prijmi cislo sektora a stopy
0348 MVI  C,80    ; logicky sektor ma 128 bytov
034A LXI  H,183F  ; adresa cieloveho buffra
034D RST  1       ; prijmi byte
034E MOV  M,A     ; uloz do pamati
034F INX  H       ; posun ukazovatel
0350 DCR  C       ; zniz pocitadlo bytov
0351 JNZ  034D    ; opakuj 128-krat
0354 RST  6       ; prijmi CRC a odosli ACK
0355 CALL 039E    ; zisti, ci je disk chraneny proti zapisu
0358 JNZ  00C0    ; ak ano, skoc do ERROR rutiny
035B CALL 0585    ; priprav pozadovany logicky sektor do pamati
035E CALL 0282    ; vypocitaj adresu buffra, kde sa nachadza sektor
0361 LXI  D,183F  ; presun prijaty logicky sektor do buffra fyzickeho sektora
0364 LDAX D
0365 MOV  M,A
0366 INX  H
0367 INX  D
0368 DCR  C
0369 JNZ  0364
036C MVI  A,FF    ; flag - aktualny fyzicky sektor v buffri je modifikovany
036E STA  183D
0371 LDA  183E    ; ak je zapnuty FAST rezim,
0374 ORA  A
0375 JNZ  0234    ; vrat sa do slucky cez odoslanie OK chyboveho kodu
0378 CALL 0382    ; inak, uloz aktualny fyzicky sektor
037B XRA  A       ; flag - aktualny fyzicky sektor je ulozeny
037C STA  183D
037F JMP  0234    ; vrat sa do slucky cez odoslanie OK chyboveho kodu


;------------------------------------------------------------------------------
; Prevedie zapis aktualneho fyzickeho sektora v pamati, ak bol modifikovany.
; vstup: -
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
; meni: vsetky
;------------------------------------------------------------------------------
0382 LDA  183D    ; je aktualny fyzicky sektor v buffri modifikovany ?
0385 ORA  A
0386 RZ           ; nie, vrat sa
0387 CALL 04A2    ; uloz fyzicky sektor
038A RZ           ; vrat sa, ak nedoslo ku chybe
038B CALL 05FD    ; zisti, ci sa nachdzas na spravnej stope; ak nie, presun sa
038E CALL 04A2    ; uloz fyzicky sektor
0391 RZ           ; vrat sa, ak nedoslo ku chybe
0392 CALL 04A2    ; uloz fyzicky sektor
0395 RZ           ; vrat sa, ak nedoslo ku chybe
0396 CALL 04A2    ; uloz fyzicky sektor
0399 RZ           ; vrat sa, ak nedoslo ku chybe
039A JMP  00C0    ; neopravitelna chyba, skok do ERROR rutiny


;------------------------------------------------------------------------------
039D DB   C9      ; seno
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; Rutina preveri, ci je disk chraneny proti zapisu.
; vstup: cislo logickeho sektora s cislom mechaniky v 7. bite na adrese 183A
;          (0 az 35 alebo 128 az 163)
;        cislo stopy na adrese 183B (0 az 79)
; vystup: NZ - disk je chraneny proti zapisu
;         Z - na disk mozno zapisovat
; meni: vsetky
;------------------------------------------------------------------------------
039E CALL 051A    ; urob prepocet cisla sektora
03A1 MVI  A,02    ; maska mechaniky
03A3 ANA  B
03A4 ORA  C       ; pridaj hlavu
03A5 STA  1816    ; cele to uloz do buffra FDC povelu SENSE DRIVE STATUS
03A8 CALL 0554    ; rozbehni pozadovanu mechaniku
03AB LXI  H,1814  ; adresa FDC povelu SENSE DRIVE STATUS
03AE CALL 01BB    ; zapis povel do FDC
03B1 LXI  H,182F  ; adresa buffru na prijatie ST3
03B4 CALL 01DA    ; precitaj status FDC
03B7 LDA  1830    ; ST3 do A
03BA ANI  40      ; odmaskuj bit WRITE PROTECT
03BC RET


;------------------------------------------------------------------------------
; povel I - vyber mechaniky a presun hlavy na stopu 0
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "I"                     | ---> |                         |
;        | cislo mechaniky         | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: -
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
03BD RST  1       ; prijmi cislo mechaniky
03BE INR  A       ; uprav hodnotu na 1 alebo 2
03BF PUSH PSW     ; odpamataj
03C0 RST  6       ; prijmi CRC a posli ACK
03C1 CALL 0382    ; uloz aktualny fyzicky sektor
03C4 POP  PSW     ; obnov cislo mechaniky
03C5 MOV  B,A     ; uloz do B
03C6 ANI  02      ; ponechaj iba 1. bit
03C8 STA  1819    ; uloz do buffra povelu RECALIBRATE - hlava na stopu 0
03CB RRC
03CC RRC          ; cislo mechaniky do 7. bitu
03CD ORI  3F      ; pridaj masku cisla logickeho sektora
03CF STA  183C    ; pre zneplatnenie naposledy pouziteho logickeho sektora
03D2 CALL 0554    ; rozbehni pozadovanu mechaniku
03D5 CALL 0488    ; posli hlavu na stopu 0
03D8 JMP  0234    ; vrat sa do slucky cez odoslanie OK chyboveho kodu


;------------------------------------------------------------------------------
; povel F - format stopy
;        |-------------------------|------|-------------------------|
;        |         PMD 85          | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "F"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK/NAK                 |
;        |                         | <--- | ERR                     |
;        |-------------------------|------|-------------------------|
; vstup: B = priebezne CRC
; vystup: naformatovana stopa, ak nedoslo ku chybe
;         skok do ERROR rutiny, ak dojde k chybe
;         ERR = 00 - OK
;               01 - Write protect
;               02 - Format error
;               03 - Verify error (chyba pri spatnom citani)
;               99 - ina chyba (FDC, PIO)
; meni: vsetky
;------------------------------------------------------------------------------
03DB CALL 0279    ; prijmi cislo sektora a stopy
03DE RST  6       ; prijmi CRC a odosli ACK
03DF CALL 051A    ; urob prepocet cisla sektora
03E2 MOV  A,E     ; cislo cylindra
03E3 STA  180D    ; do buffra povelu FORMAT TRACK
03E6 MVI  A,02    ; maska mechaniky
03E8 ANA  B
03E9 STA  1819    ; do buffra povelu FORMAT TRACK
03EC ORA  C       ; pridaj hlavu uloz
03ED STA  180C    ; do buffra povelu READ/WRITE DATA
03F0 STA  181F    ; do buffra povelu FORMAT A TRACK
03F3 STA  181C    ; do buffra povelu READ ID
03F6 MOV  A,C     ; cislo hlavy
03F7 RRC
03F8 RRC
03F9 ANI  01      ; presun do 0. bitu a
03FB STA  180E    ; do buffra povelu READ/WRITE DATA
03FE LXI  H,1A00  ; adresa buffru, kde sa ulzia parametre formatovanej stopy
0401 MOV  D,A     ; cislo hlavy do D
0402 MOV  C,A     ; aj C - zbytocne
0403 LXI  B,0656  ; adresa poradia cisel (ID) sektorov pre hlavu 0
0406 MOV  A,D     ; je to hlava 0 ?
0407 ANA  A
0408 JZ   040E    ; ano skoc dalej
040B LXI  B,0653  ; adresa poradia cisel (ID) sektorov pre hlavu 1
040E MVI  A,FF    ; zneplatni naposledy pouzity logicky sektor a mechaniku
0410 STA  183C
; naplnenie buffra parametrami formatovania v poradi - pre kazdy sektor v stope
; cylinder, hlava, cislo (ID) sektora, velkost
0413 MVI  A,09    ; 9 sektorov na stopu
0415 PUSH PSW     ; odpamataj pocitadlo
0416 MOV  M,E     ; cylinder
0417 INX  H
0418 MOV  M,D     ; hlava
0419 INX  H
041A LDAX B       ; cislo sektora
041B MOV  M,A
041C INX  H
041D MVI  M,02    ; velkost sektora - 512 bytov
041F INX  H
0420 INX  B       ; posun v tabulke poradia ID sektorov
0421 POP  PSW
0422 DCR  A       ; opakuj pre vsetky sektory
0423 JNZ  0415
0426 CALL 039E    ; zisti, ci je disk chraneny proti zapisu
0429 JZ   0431    ; ak nie, skoc dopredu
042C MVI  A,01    ; inak vrat chybovy kod 01 - Write Protect
042E JMP  0235    ; vrat sa do slucky s odoslanim chyboveho kodu

0431 CALL 061F    ; rozbehni mechaniku a nastav pozadovanu stopu
0434 MVI  A,41    ; nastavenie rezimu DMA - zapnuty DMA kanal 0, automaticke
0436 OUT  48      ;   vypnutie DMA kanalu 0 po dopocitani (TC STOP)
0438 MVI  A,00    ; adresa parametrov formatovania (1A00) do registra adresy
043A OUT  40      ;   DMA kanalu 0
043C MVI  A,1A
043E OUT  40
0440 MVI  A,23    ; pocet (36 - 1) do registra pocitadla DMA kanalu 0
0442 OUT  41
0444 MVI  A,80    ; v bitoch 7 a 6 je nastavena operacia citania MEM -> FDC
0446 OUT  41
0448 LXI  H,181D  ; adresa buffra FDC povelu FORMAT A TRACK
044B CALL 01BB    ; zapis povel do FDC - sformatuj stopu
044E CALL 04DE    ; pockaj si na vysledok operacie
0451 JZ   0459    ; a ak je vsetko OK, skoc dalej
0454 MVI  A,02    ; inak, vrat chybovy kod 02 - Foramat Error
0456 JMP  0235    ; vrat sa do slucky s odoslanim chyboveho kodu
; verifikacia
0459 XRA  A       ; A je pocitadlo sektorov v stope
045A PUSH PSW
045B MOV  E,A     ; vypocitaj offset do tabulky ID sektorov
045C MVI  D,00
045E LXI  H,0656
0461 DAD  D
0462 MOV  A,M     ; ID sektora
0463 DCR  A       ; uprav ID sektora na logicky rozsah 0 az 32 (s krokom 4)
0464 RLC
0465 RLC
0466 MOV  D,A     ; uloz do D
0467 LDA  183A    ; aktualne cislo logickeho sektora - kvoli cislu mechaniky
046A ANI  80      ; ponechaj iba bit mechaniky
046C ORA  D       ; a pridaj cislo logickeho sektora
046D STA  183A    ; uloz
0470 CALL 04A9    ; pokus sa precitat sektor
0473 JZ   047B    ; ak to bolo v poriadku, skoc dalej
0476 MVI  A,03    ; inak, vrat chybovy kod 03 - Verify Error (chyba pri citani)
0478 JMP  0235    ; vrat sa do slucky s odoslanim chyboveho kodu

047B POP  PSW     ; obnov pocitadlo sektorov
047C INR  A       ; zvys pocitadlo
047D CPI  09
047F JNZ  045A    ; ak ci este nepresiel vsetky, opakuj
0482 CALL 05E5    ; odzalohuj premenne
0485 JMP  0234    ; vrat sa do slucky cez odoslanie OK chyboveho kodu


;------------------------------------------------------------------------------
; Rutina posle hlavu domov - na stopu 0.
; vstup: pripraveny buffer pre FDC povel RECALIBRATE na adrese 1817
; vystup: -
; meni: vsetky
;------------------------------------------------------------------------------
0488 LXI  D,1817  ; adresa buffra FDC povelu RECALIBRATE
048B LXI  H,0200  ; adresa prerusovacej rutiny
048E CALL 01F9    ; nastav adresu rutiny a zapis FDC povel
0491 LDA  1819    ; zisti, o ktoru mechaniku sa jedna
0494 ANA  A
0495 MVI  A,00    ; stopa 0
0497 JNZ  049E
049A STA  1835    ; uloz pre mechaniku A
049D RET

049E STA  1836    ; uloz pre mechaniku B
04A1 RET


;------------------------------------------------------------------------------
; Zapis fyzickeho sektoru.
; vstup: cislo logickeho sektora s cislom mechaniky v 7. bite na adrese 183A
;          (0 az 35 alebo 128 az 163)
;        cislo stopy na adrese 183B (0 az 79)
; vystup: A = 0 - OK, >0 - chyba
; meni: vsetky
;------------------------------------------------------------------------------
04A2 MVI  C,80    ; kod operacie pre DMA - FDC <- MEM
04A4 MVI  A,45    ; kod povelu pre FDC - zapis
04A6 JMP  04AD    ; skoc do spolocnej casti pre zapis/citanie fyzickeho sektoru


;------------------------------------------------------------------------------
; Citanie fyzickeho sektoru.
; vstup: cislo logickeho sektora s cislom mechaniky v 7. bite na adrese 183A
;          (0 az 35 alebo 128 az 163)
;        cislo stopy na adrese 183B (0 az 79)
; vystup: A = 0 - OK, >0 - chyba
; meni: vsetky
;------------------------------------------------------------------------------
04A9 MVI  A,46    ; kod povelu pre FDC - MEM <- FDC
04AB MVI  C,40    ; kod operacie pre DMA - citanie
04AD STA  180B    ; uloz kod FDC do buffra povelu
04B0 PUSH B       ; odpamataj kod operacie pre DMA
04B1 CALL 051A    ; urob prepocet cisla sektora
04B4 MOV  A,D
04B5 STA  180F    ; uloz cislo sektora do buffra R/W povelu FDC
04B8 MOV  A,E
04B9 STA  180D    ; uloz cislo cylindra do buffra R/W povelu FDC
04BC MVI  A,02    ; maska na cislo mechaniky
04BE ANA  B
04BF STA  1819    ; uloz cislo mechaniky do buffra RECALIBRATE povelu FDC
04C2 ORA  C       ; pridaj cislo hlavu
04C3 STA  180C    ; uloz cislo hlavy a mechaniky do buffra R/W povelu FDC
04C6 STA  181C    ; uloz cislo hlavy a mechaniky do buffra READ ID povelu FDC
04C9 MOV  A,C     ; uprav cislo hlavy na hodnotu 0 alebo 1
04CA RRC
04CB RRC
04CC ANI  01      ; odmaskuj
04CE STA  180E    ; uloz cislo hlavy do buffra R/W povelu FDC
04D1 CALL 0551    ; rozbehni pozadovanu mechaniku
04D4 POP  B       ; obnov operaciu DMA
04D5 CALL 0503    ; zahaj DMA prenos
04D8 LXI  H,180A  ; adresa povelu R/W pre FDC
04DB CALL 01BB    ; zapis povel do FDC

; tu sa skace po operaciach s diskom pre ziskanie vysledku operacie
04DE CALL 01FC    ; pockaj si na prerusenie od DMA
04E1 LXI  H,1824  ; adresa buffru pre ulozenie vysledku z FDC
04E4 CALL 01DA    ; citaj vysledok z FDC
04E7 LXI  D,FFE8  ; DE = -24
04EA LHLD 1831    ; timeout zastavenia mechaniky A
04ED DAD  D       ; odpocitaj 24
04EE SHLD 1831    ; uloz naspat
04F1 LHLD 1833    ; timeout zastavenia mechaniky B
04F4 DAD  D       ; odpocitaj 24
04F5 SHLD 1833    ; uloz naspat
04F8 LXI  H,1825  ; adresa status registra ST0
04FB MOV  A,M     ; ST0 do A
04FC ANI  F8      ; vynuluj bity mechaniky a hlavy
04FE INX  H
04FF ORA  M       ; pridaj ST1
0500 INX  H
0501 ORA  M       ; pridaj ST2
0502 RET          ; vrat sa vysledkom operacie


;------------------------------------------------------------------------------
; Inicializacia a spustenie DMA operacie.
; vstup: C = kod operacie v bitoch 7 a 6 => 10 - citanie, 01 - zapis
; vystup: -
; meni: AF, HL
;------------------------------------------------------------------------------
0503 MVI  A,41    ; nastavenie rezimu DMA - zapnuty DMA kanal 0, automaticke
0505 OUT  48      ;   vypnutie DMA kanalu 0 po dopocitani (TC STOP)
0507 LXI  H,18BF  ; adresa buffra do registra adresy DMA kanalu 0
050A MOV  A,L
050B OUT  40
050D MOV  A,H
050E OUT  40
0510 MVI  A,FF    ; pocet (512 - 1) do registra pocitadla DMA kanalu 0
0512 OUT  41
0514 MVI  A,01
0516 ORA  C       ; k vyssiemu bytu pocitadla pridaj operaciu (citanie/zapis)
0517 OUT  41
0519 RET


;------------------------------------------------------------------------------
; Prepocet logickeho cislovania sektora a stopy na fyzicke.
; vstup: cislo logickeho sektora s cislom mechaniky v 7. bite na adrese 183A
;          (0 az 35 alebo 128 az 163)
;        cislo stopy na adrese 183B (0 az 79)
; vystup: B = mechanika (1 pre A, 2 pre B)
;         C = cislo hlavy v 2. bite (0 alebo 4)
;         E = cislo cylindra (0 az 39)
;         D = cislo (ID) sektora (1 az 9)
;         L = cislo logickeho sektora v ramci fyzickeho sektora (0 az 3)
; meni: AF, BC, DE, L
;------------------------------------------------------------------------------
051A LDA  183B    ; cislo stopy do A
051D ANA  A       ; zhod CY
051E RAR          ; CYL = TRK / 2, CY = hlava
051F MOV  E,A     ; cislo cylindra de E,
0520 RAL          ; hlava do 2. bitu
0521 RAL
0522 RAL
0523 ANI  04      ; odmaskuj
0525 MOV  C,A     ; a uloz do C
0526 LDA  183A    ; cislo sektora do A
0529 MOV  D,A     ; a do D
052A ANI  03      ; odmaskuj si cislo logickeho sektora v ramci fyzickeho sekt.
052C MOV  L,A     ; uloz do L (512 = 4 * 128)
052D MOV  A,D     ; v 7. bite je cislo mechaniky
052E ANI  80      ; odmaskuj
0530 MVI  B,01    ; mechnika A
0532 JZ   0536
0535 INR  B       ; mechanika B
0536 MOV  A,D     ; vypocitaj cislo fyzickeho sektora v stope
0537 RAR          ; SEC = LSEC / 4
0538 RAR
0539 ANI  0F      ; odmaskuj
053B MOV  D,A     ; uloz do D
053C INR  D       ; a uprav na rozsah 1 az 9
053D RET


;------------------------------------------------------------------------------
; Rutina SEEK - posun hlavy na pozadovanu storpu.
; vstup: pripravene hlava a mechanika v povele pre FDC na adrese 1808
;        pripravene cislo cylindra v povele pre FDC na adrese 1809
; vystup: hlava na pozadovanej stope, ak nedojde ku chybe
; meni: vsetky
;------------------------------------------------------------------------------
053E LXI  D,1806  ; adresa povelu FDC
0541 LXI  H,0200  ; adresa prerusovacej rutiny
0544 CALL 01F9    ; preved posunutie hlavy na pozadovanu stopu
0547 LXI  H,03E8  ; zdrzanie asi 11,7 ms
054A DCX  H
054B MOV  A,H
054C ORA  L
054D JNZ  054A
0550 RET


;------------------------------------------------------------------------------
; Rutina prevedie vyber pozadovanej mechaniky a zapne jej motor.
; Zaroven inicializuje timer automatickeho vypnutia motora mechaniky.
; vstup: cislo logickeho sektora s cislom mechaniky v 7. bite na adrese 183A
;          (0 az 35 alebo 128 az 163)
;        cislo stopy na adrese 183B (0 az 79)
; vystup: rozbehnuta pozadoana mechanika
; meni: vsetky
;------------------------------------------------------------------------------
0551 CALL 051A    ; urob prepocet cisla sektora - B = cislo mechaniky
0554 DCR  B       ; je to mechanika A ?
0555 LXI  H,07D0  ; do HL pocitadlo - 2000
0558 JNZ  0563    ; skoc dalej, ak je to mechanika B
055B SHLD 1831    ; inicializuj pocitadlo mechaniky A
055E MVI  B,50    ; maska pre mechaniku A
0560 JMP  0568    ; pokracuj dalej

0563 SHLD 1833    ; inicializuj pocitadlo mechaniky B
0566 MVI  B,A0    ; maska pre mechaniku B
0568 LDA  1837    ; stav portu mechanik
056B ANI  30      ; ponechaj iba stav MO1 a MO0
056D MOV  C,A     ; uloz stav motorov do C
056E ORA  B       ; pridaj DSx a MOx podla vybranej mechaniky
056F STA  1837    ; uloz stav
0572 ORI  08      ; pridaj bit ENA
0574 OUT  E0      ; a zapis na port
0576 MOV  A,C     ; ak sa uz motor pozadovanej mechaniky toci,
0577 ANA  B
0578 RNZ          ; vrat sa
0579 LXI  B,1F40  ; inak pockaj asi 235 ms
057C XTHL         ; kym sa rozbehne motor pozadovanej mechaniky
057D XTHL
057E DCX  B
057F MOV  A,B
0580 ORA  C
0581 JNZ  057C
0584 RET


;------------------------------------------------------------------------------
; Rutina pripravi pozadovany logicky sektor do pamati.
; Ak sa zisti, ze pozadovany logicky sektor nie je v pamati a fyzicky sektor
; v pamati bol modifikovany, ten sa najprv ulozi na disk. Nasledne sa precita
; fyzicky sektor, do ktoreho patri pozadovany logicky sektor.
; vstup: systemove premenne
; vystup: pripraveny logicky sektor v pamati
; meni: vsetky
;------------------------------------------------------------------------------
0585 LDA  183C    ; povodne cislo logickeho sektoru s cislom aktualnej mechaniky
0588 MOV  E,A     ; do E
0589 ANA  A
058A LDA  1835    ; stopa, nad ktorou je hlava mechaniky A
058D JP   0593    ; ak je to mechanika A, skoc dopredu
0590 LDA  1836    ; stopa, nad ktorou je hlava mechaniky B
0593 MOV  D,A     ; stopa, nad ktorou je hlava aktualnej mechaniky do D
0594 LDA  183A    ; pozadovane cislo logickeho sektora a mechaniku
0597 XRA  E       ; porovnaj s povodnou hodnotou
0598 ANI  FC      ; do uvahy zober iba zmenu z pohladu fyzickeho sektora
059A JNZ  05A2    ; bola zmena, pozadovany logicky sektor nie je v pamati
059D LDA  183B    ; sektor a mechanika sa zhoduje,
05A0 CMP  D       ; porovnaj stopu
05A1 RZ           ; vrat sa, pozadovany logicky sektor je v pamati
05A2 LDA  183D    ; je aktualny fyzicky sektor v buffri modifikovany ?
05A5 ANA  A
05A6 JZ   05B8    ; nie, skoc dopredu
05A9 LHLD 183A    ; ak ano, odpamataj si pozadovany sektor, mechaniku a stopu
05AC PUSH H
05AD XCHG         ; priprav povodny sektor, mechaniku a stopu
05AE SHLD 183A
05B1 CALL 0382    ; uloz aktualny fyzicky sektor
05B4 POP  H       ; obnov pozadovany sektor, mechaniku a stopu
05B5 SHLD 183A
05B8 XRA  A       ; flag - aktualny fyzicky sektor je ulozeny
05B9 STA  183D
05BC MVI  A,FF    ; zneplatni naposledy pouzity logicky sektor a mechaniku
05BE STA  183C
05C1 CALL 061F    ; rozbehni mechaniku a nastav pozadovanu stopu
05C4 CALL 04A9    ; pokus sa precitat sektor
05C7 JZ   05E5    ; ak je OK, skoc na zalohovanie premennych
05CA CALL 05FD    ; zisti, ci sa nachdzas na spravnej stope; ak nie, presun sa
05CD CALL 04A9    ; pokus sa precitat sektor
05D0 JZ   05E5    ; ak je OK, skoc na zalohovanie premennych
05D3 CALL 04A9    ; pokus sa precitat sektor
05D6 JZ   05E5    ; ak je OK, skoc na zalohovanie premennych
05D9 CALL 04A9    ; pokus sa precitat sektor
05DC JZ   05E5    ; ak je OK, skoc na zalohovanie premennych
05DF CALL 04A9    ; pokus sa precitat sektor
05E2 JNZ  00C0    ; pri chybe skoc do ERROR rutiny
                  ; OK, pokracuj zalohovanim premennych

;------------------------------------------------------------------------------
; Rutina odpamata aktualne cislo sektora a stopy do zaloznych premennych.
; vstup: -
; vystup: -
; meni: vsetky
;------------------------------------------------------------------------------
05E5 LDA  183A    ; aktualne cislo logickeho sektora a mechaniky
05E8 STA  183C    ; uloz
05EB CALL 051A    ; urob prepocet cisla sektora - B = cislo mechaniky
05EE LDA  183B    ; aktualne cislo stopy, kde sa nachadza hlava
05F1 DCR  B
05F2 JNZ  05F9
05F5 STA  1835    ; uloz pre mechaniku A
05F8 RET

05F9 STA  1836    ; uloz pre mechaniku B
05FC RET


;------------------------------------------------------------------------------
; Rutina zisti, ci sa hlava aktialnej mechaniky nachdza na spravnej stope
; precitanim ID sektora.
; Ak nie, posle hlavu najprv domov (na stopu 0) a potom urobi SEEK na
; pozadovanu stopu.
; vstup: systemove premenne pre aktualnu mechaniku
; vystup: hlava na pozadovanej stope, ak nedojde ku chybe
; meni: vsetky
;------------------------------------------------------------------------------
05FD CALL 051A    ; urob prepocet cisla sektora
0600 PUSH D       ; odpamataj si cislo cylindra
0601 LXI  H,181A  ; buffer FDC povelu READ ID
0604 CALL 01BB    ; zapis povel do FDC
0607 CALL 04DE    ; pockaj si na vysledok operacie
060A POP  D       ; obnov cislo cylindra
060B JNZ  0613    ; ak doslo ku chybe, skoc poslat hlavu domov
060E LDA  1828    ; cislo cylindra s resultu
0611 CMP  E       ; porovnaj s pozadovanou
0612 RZ           ; vrat sa, ak je to v poriadku
0613 LXI  D,1817  ; adresa buffra FDC povelu RECALIBRATE
0616 LXI  H,0200  ; adresa prerusovacej rutiny
0619 CALL 01F9    ; nastav adresu rutiny a zapis FDC povel
061C JMP  053E    ; posun hlavu na pozadovanu stopu


;------------------------------------------------------------------------------
; Rutina rozbehne mechaniku a pripadne aj nastavi pozadovanu stopu.
; vstup: systemove premenne pre aktualnu mechaniku
; vystup: hlava na pozadovanej stope, ak nedojde ku chybe
; meni: vsetky
;------------------------------------------------------------------------------
061F CALL 0551    ; rozbehni pozadovanu mechaniku
0622 CALL 051A    ; urob prepocet cisla sektora
0625 MVI  A,02    ; maska cisla mechaniky
0627 ANA  B
0628 STA  1808    ; uloz do bufra pre FDC povel SEEK
062B STA  1819    ; uloz do bufra pre FDC povel RECALIBRATE
062E ORA  C       ; pridaj hlavu
062F STA  1816    ; a uloz do bufra pre FDC povel SENSE DRIVE STATUS
0632 MOV  A,E     ; cislo cylindra
0633 STA  1809    ; uloz do bufra pre FDC povel SEEK
0636 LDA  1836    ; cislo stopy, nad ktorou je hlava mechaniky B
0639 DCR  B       ; ide o mechaniku B ?
063A JNZ  0640    ; ano, skoc dalej
063D LDA  1835    ; cislo stopy, nad ktorou je hlava mechaniky A
0640 ANA  A       ; zhod CY
0641 RAR          ; preved stopu na cylinder
0642 CMP  E       ; si na spravnej stope ?
0643 CNZ  053E    ; ak nie, posun hlavu na pozadovanu stopu
0646 RET


;------------------------------------------------------------------------------
; Command - SPECIFY
; - Parametre:
;   1. SRT = 0D, HUT = 0F - Step Rate Time - 3 ms, Head Unload Time - 240 ms
;   2. HLT = 01, ND = 0 - Head Load Time - 2 ms, DMA mode
;------------------------------------------------------------------------------
0647 DB   03,03,DF,02


;------------------------------------------------------------------------------
; Command - RECALIBRATE
; posle hlavu mechaniky A na stopu 0
; - Parametre:
;   1. pozadovana mechanika: X X X X X 0 US1 US0 - 00
;------------------------------------------------------------------------------
064B DB   02,07,00


;------------------------------------------------------------------------------
; Command - RECALIBRATE
; posle hlavu mechaniky B na stopu 0
; - Parametre:
;   1. pozadovana mechanika: X X X X X 0 US1 US0 - 02
;------------------------------------------------------------------------------
064E DB   02,07,02


;------------------------------------------------------------------------------
; Command - SENSE INTERRUPT STATUS
; zistenie stavu prerusenia FDC
; - Vysledok:
;   1. ST0
;   2. aktualne cislo cylindra
;------------------------------------------------------------------------------
0651 DB   01,08


;------------------------------------------------------------------------------
; Tabulka poradia sektorov pri formatovani. Pre hlavu 0 zacina na adrese 0656
; a pre hlavu 1 na adrese 0653
;------------------------------------------------------------------------------
0653 DB   02,05,08
0656 DB   03,06,09,01,04,07
065C DB   02,05,08


;------------------------------------------------------------------------------
; Nasledujuci kus kodu a data su po starte presunute do RAM na adresu 1800
;------------------------------------------------------------------------------
065F JMP  0519
0662 JMP  01FE

0665 DB   03,0F,00,00,09,00,00,00
066D DB   00,01,02,09,2A,FF,02,04
0675 DB   00,02,07,00,02,4A,00,06
067D DB   4D,00,02,09,50,E5,07,00
0685 DB   00,00,00,00,00,00,02,00
068D DB   00,01,00,FF,01,FF,01,00
0695 DB   00,30,00,00,00,00,FF,00
069D DB   00


;******************************************************************************
; Nasledujuci kod a udaje su prekopirovane z ROM do RAM, kvoli potrebe
; modifikacie za behu programu.
;******************************************************************************

;------------------------------------------------------------------------------
; Skok na RET.
; Je to volane po prijati neznameho kodu povelu od PMD 85.
;------------------------------------------------------------------------------
1800 JMP  0519    ; skok na RET


;------------------------------------------------------------------------------
; Vstupny bod pre skok do prerusovacej rutiny.
; Adresa skoku je podla potreby modifikovana na adresu 01FE alebo 0200.
;------------------------------------------------------------------------------
1803 JMP  01FE    ; akok do prerusovacej rutiny


;------------------------------------------------------------------------------
; Command - SEEK
; presun hlavy na pozadovanu stopu pozadovanej mechaniky
; - Parametre:
;   1. hlava a mechanika: X X X X X HD US1 US0
;   2. cislo cylindra
;------------------------------------------------------------------------------
1806 DB   03,0F,00,00


;------------------------------------------------------------------------------
; Command - WRITE DATA (45) alebo READ DATA (46)
; zapis alebo precitanie sektora z disku v MFM mode
; - Parametre:
;   1. hlava a mechanika: X X X X X HD US1 US0
;   2. cislo cylindra
;   3. cislo hlavy
;   4. cislo sektora
;   5. velkost sektora (128 << N) - N = 02 => 512 bytov
;   6. cislo posledneho sektora v stope - EOT = 09
;   7. velkost medzery Gap 3 - GPL = 2A
;   8. velkost dat - DTL = FF (nepodstatne, kedze mame sektor odlzke 512 bytov)
; - Vysledok:
;   1. ST0
;   2. ST1
;   3. ST2
;   4. cislo cylindra
;   5. cislo hlavy
;   6. cislo sektora
;   7. velkost sektora
;------------------------------------------------------------------------------
180A DB   09,00,00,00,00,01,02,09,2A,FF


;------------------------------------------------------------------------------
; Command - SENSE DRIVE STATUS
; zistenie stavu danej mechaniky
; - Parametre:
;   1. hlava a mechanika: X X X X X HD US1 US0
; - Vysledok:
;   1. ST3 - v 6. bite je Write protect bit
;------------------------------------------------------------------------------
1814 DB   02,04,00


;------------------------------------------------------------------------------
; Command - RECALIBRATE
; posle hlavu pozadovanej mechaniky na stopu 0
; - Parametre:
;   1. pozadovana mechanika: X X X X X 0 US1 US0
;------------------------------------------------------------------------------
1817 DB   02,07,00


;------------------------------------------------------------------------------
; Command - READ ID
; citanie ID sektora v MFM mode
; - Parametre:
;   1. hlava a mechanika: X X X X X HD US1 US0
; - Vysledok:
;   1. ST0
;   2. ST1
;   3. ST2
;   4. cislo cylindra
;   5. cislo hlavy
;   6. cislo sektora
;   7. velkost sektora
;------------------------------------------------------------------------------
181A DB   02,4A,00


;------------------------------------------------------------------------------
; Command - FORMAT A TRACK
; formatovanie stopy v MFM mode
; - Parametre:
;   1. hlava a mechanika: X X X X X HD US1 US0
;   2. velkost sektora (128 << N) - N = 02 => 512 bytov
;   3. pocet sektorov na stopu - SC = 09
;   4. velkost medzery Gap 3 - GPL = 50
;   5. vyplnovaci byte - D = E5
; - Vysledok:
;   1. ST0
;   2. ST1
;   3. ST2
;   4. cislo cylindra
;   5. cislo hlavy
;   6. cislo sektora
;   7. velkost sektora
;------------------------------------------------------------------------------
181D DB   06,4D,00,02,09,50,E5


;------------------------------------------------------------------------------
; Buffer pre precitanie 7 bytoveho vysledku operacie z FDC
; - Vysledok:
;   1. ST0
;   2. ST1
;   3. ST2
;   4. cislo cylindra
;   5. cislo hlavy
;   6. cislo sektora
;   7. velkost sektora
;------------------------------------------------------------------------------
1824 DB   07,00,00,00,00,00,00,00


;------------------------------------------------------------------------------
; Buffer pre precitanie 2 bytoveho vysledku operacie z FDC
; - Vysledok:
;   1. ST0
;   2. cislo aktualneho cylindra
;------------------------------------------------------------------------------
182C DB   02,00,00


;------------------------------------------------------------------------------
; Buffer pre precitanie 1 bytoveho vysledku operacie z FDC
; - Vysledok:
;   1. ST3 - v 6. bite je Write protect bit
;------------------------------------------------------------------------------
182F DB   01,00


;------------------------------------------------------------------------------
; Systemove premenne
;------------------------------------------------------------------------------
1831 DW   01FF    ; pocitadlo timeoutu pre vypnutie motora mechaniky A
1833 DW   01FF    ; pocitadlo timeoutu pre vypnutie motora mechaniky B

1835 DB   00      ; cislo stopy, nad ktorou sa prave nachdza hlava mechaniky A
1836 DB   00      ; cislo stopy, nad ktorou sa prave nachdza hlava mechaniky B
1837 DB   30      ; stav portu ovladania mechanik
1838 DB   00      ; nepouzite
1839 DB   00      ; nepouzite
183A DB   00      ; aktualne cislo log. sektora, v 7. bite je cislo mechaniky
183B DB   00      ; aktualne cislo stopy
183C DB   FF      ; naposledy pouzity logicky sektor a mechanika
183D DB   00      ; aktualny fyzicky sektor je: 00 - ulozeny, FF - modifikovany
183E DB   00      ; 00 - SLOW rezim, FF - FAST rezim

183F DS   80      ; buffer 128 bytov pre jeden logicky sektor
18BF DS   0201    ; buffer 513 bytov pre jeden fyzicky sektor

1A00 DS   24      ; buffer pre parametre formatovanej stopy

1AC0              ; volna pamat RAM asi 256 bytov pre uzivatela

1BFF              ; od tejto adresy k nizsim adresam je umiesteneny zasobnik CPU

;------------------------------------------------------------------------------
