;------------------------------------------------------------------------------
; Disassembling a komentar Roman Brik - 2005/2006/2009 (Revizia 8)
;------------------------------------------------------------------------------
; Cast monitora PMD 85-3 pre obsluhu disketovej jednotky PMD 32
; a zabezpecenie konzolovych operacii pre BIOS Mikrosu.
;------------------------------------------------------------------------------
; PMD 32 je k PMD 85-3 pripojena cez GPIO/0.
; Vyuziva port A 8255 ako obojstrannu datovu zbernicu.
;
; Komunikacia prebieha tak, ze PMD 85-3 posiela povely a PMD 32 na ne odpoveda.
; Zahajenie komunikacie je reprezentovane vymenou prezentacnych bytov AAh.
;
; POVELY:
; "B" - vyziadanie BOOT sektora, jeho nahratie na adresu 0080h a spustenie
; "Q" - precitanie jedneho logickeho sektora 128 bytov z diskety
; "T" - zapisanie jedneho logickeho sektora 128 bytov na disketu
; "I" - volba mechaniky a posun hlavy na stopu 0
; "*" - rezim FAST
; "@" - rezim SLOW


;------------------------------------------------------------------------------
; premenne pouzivane diskovou castou
BFF6 DS   2       ; adresa v pamati
BFF8 DS   1       ; cislo mechaniky
BFFC DS   1       ; sektor (0 az 35) + 7. bit je nastaveny pre mechaniku B
BFFE DS   1       ; stopa

; premenne v zapisniku monitora pouzivane konzolovymi operaciami
C03A DS   1       ; kod farby
C03E DS   2       ; adresa kurzora
C0F4 DS   1       ; pocet prenasanych mikroriadkov pri rolovani
C0FC DS   2       ; sirka riadku - offset medzi dvoma riadkami
C134 DS   1       ; kod stlacenej klavesy
C13E DS   2       ; odpamatanie SP
C33A DS   1       ; povodna hodnota poctu prenasanych mikroriadkov pri rolovani
C33C DS   1       ; priznak ESC sekvencie
C33D DS   1       ; cislo riadku pre ESC sekvenciu


;------------------------------------------------------------------------------
; Rutina je volana pri starte systemu po teste pamati a pred zavedenim
; ROM-modulu.
; Pokusi sa "naviazat spojenie" s PMD 32, co je reprezentovane vymenou
; prezentacnych bytov AAh. Ak je spojenie naviazane vyziada si BOOT sektor
; z vlozenej diskety, ten ulozi od adresy 0080h a spusti tento BOOT kod.
; Pri neuspechu sa vrati a system pokracuje zavedenim ROM-modulu, ak je vlozeny.
; vstup: -
; vystup: spustenie BOOT kodu alebo navrat pri neuspechu
; meni: vsetky
;------------------------------------------------------------------------------
F91F MVI  B,05    ; 4 pokusy
F921 LXI  H,0080  ; adresa buffru
F924 SHLD BFF6    ; uloz na adresu BFF6
F927 DCR  B       ; zniz pocitadlo pokusov
F928 JNZ  F938    ; a skoc, ak este neboli vsetky
F92B MVI  A,AD    ; inak inicializuj GPIO 8255
F92D OUT  4F      ; SK A: MOD 1, A-OUT, C5,C4-IN,  SK B: MOD 1, B-OUT
F92F MVI  A,0D    ; 1->C6 povolenie prerusovacieho signalu INTR pre vystup A
F931 OUT  4F
F933 MVI  A,05    ; 1->C2 povolenie prerusovacieho signalu INTR pre vystup B
F935 OUT  4F
F937 RET

F938 PUSH B       ; uchovaj pocitadlo
F939 CALL FA61    ; zisti, ci je pripojena disketova jednotka a precitaj BOOT
F93C POP  B       ; obnov pocitadlo
F93D JC   F927    ; ak doslo ku chybe, skoc do slucky pre dalsi pokus
F940 JMP  0080    ; inak, vykonaj nacitany kod na adrese 0080


;------------------------------------------------------------------------------
; Precita byte z PMD 32 a zaroven modifikuje CRC.
; vstup: -
; vystup: CY = 1 - chyba
;         CY = 0 - A = C = precitany byte, B = modifikovane CRC
; meni: AF, BC
;------------------------------------------------------------------------------
F943 CALL F961    ; je platny byte na vstupe?
F946 RC           ; nie, vrat sa s chybou
F947 IN   4C      ; precitaj byte
F949 MOV  C,A     ; odloz si ho do C
F94A XRA  B       ; modifikuj CRC
F94B MOV  B,A     ; a uloz ho naspat do B
F94C MOV  A,C     ; vrat nacitany byte do A
F94D RET


;------------------------------------------------------------------------------
; Posle povel do PMD 32. Pred poslanim povelu sa najprv preveri, ci PMD 32
; komunikuje.
; vstup: A = povel
; vystup: CY = 1 - chyba
;         CY = 0 - OK, B = A = modifikovane CRC
; meni: AF, B
;------------------------------------------------------------------------------
F94E MOV  B,A     ; uloz posielany byte do B
F94F PUSH B       ; BC na zasobnik
F950 CALL F9B9    ; zisti, ci bol prijaty prezentacny byte AA, ak nie, posli ho
F953 POP  B       ; obnov BC
F954 RC           ; vrat sa, ak doslo ku chybe
F955 MOV  A,B     ; vrat posielany byte do A
F956 MVI  B,00    ; vynuluj CRC
; pokracuj dalej


;------------------------------------------------------------------------------
; Posle byte do PMD 32
; vstup: A = posielany byte
; vystup: CY = 1 - chyba
;         CY = 0 - OK, B = A = modifikovane CRC
; meni: AF, B
;------------------------------------------------------------------------------
F958 CALL F961    ; zisti, ci je volny vystup
F95B RC           ; vrat sa, ak doslo ku timeoutu
F95C OUT  4C      ; posli byte na vystup
F95E XRA  B       ; modofikuj CRC
F95F MOV  B,A     ; a uloz ho naspat do B
F960 RET


;------------------------------------------------------------------------------
; Zisti, ci je nastavene INTRA.
; INTRA=1, ak je platny byte na vstupe, alebo bol precitany byte z vystupu
; vstup: -
; vystup: CY = 1 - timeout
;         CY = 0 - OK
; meni: CY
;------------------------------------------------------------------------------
F961 PUSH PSW     ; uloz AF a BC
F962 PUSH B
F963 MVI  C,00    ; timeout asi 5 ms
F965 DCR  C       ; zniz pocitadlo
F966 JNZ  F96D    ; a ak nie je nulove, skoc otestovat INTRA
F969 POP  B       ; doslo k timeoutu
F96A POP  PSW     ; obnov BC a AF
F96B STC          ; nastav CY ako priznak chyby
F96C RET          ; vrat sa s chybou

F96D IN   4E      ; zisti stav INTRA
F96F ANI  08
F971 JZ   F965    ; ak je INTRA=0, skoc do slucky
F974 POP  B       ; inak obnov BC a AF
F975 POP  PSW
F976 ORA  A       ; vynuluj CY
F977 RET


;------------------------------------------------------------------------------
; Povoli prerusenie pre vstup na porte A
; vstup: -
; vystup: povolene prerusenie pre vstup bytu
; meni: A
;------------------------------------------------------------------------------
F978 MVI  A,09    ; 1->C4 povolenie prerusovacieho signalu INTRA pre vstup A
F97A OUT  4F
F97C MVI  A,0C    ; 0->C6 zakazanie prerusovacieho signalu INTRA pre vystup A
F97E OUT  4F
F980 RET


;------------------------------------------------------------------------------
; Povoli prerusenie pre vystup portu A
; vstup: -
; vystup: povolene prerusenie pre vystup bytu
; meni: A
;------------------------------------------------------------------------------
F981 MVI  A,08    ; 0->C4 zakazanie prerusovacieho signalu INTRA pre vstup A
F983 OUT  4F
F985 MVI  A,0D    ; 1->C6 povolenie prerusovacieho signalu INTRA pre vystup A
F987 OUT  4F
F989 RET


;------------------------------------------------------------------------------
; Zisti, ci je pripojena PMD 32 a ci odpoveda prezentacnym bytom.
; vstup: -
; vystup: CY = 1 - chyba, timeout, PMD 32 neodpoveda prezentacnym bytom
; meni: AF, DE, B
;------------------------------------------------------------------------------
F98A MVI  A,C4    ; inicializuj GPIO 8255
F98C OUT  4F      ; SK A: MOD 2, A-IN/OUT,  SK B: MOD 1, B-OUT
F98E MVI  A,05    ; 1->C2 povolenie prerusovacieho signalu INTRB pre vystup B
F990 OUT  4F
F992 LXI  D,1388  ; timeout asi 250 ms
F995 CALL F978    ; povol prerusenie pre vstup
F998 DCX  D       ; zniz pocitadlo timeoutu
F999 MOV  A,D     ; a testuj ho na nulu
F99A ORA  E
F99B STC          ; nastav CY ako priznak timeoutu
F99C RZ           ; vrat sa s CY, ak cas vyprsal
F99D IN   4E      ; cakaj na signal INTRA, teda ci su na porte A platne data
F99F ANI  08
F9A1 JZ   F998    ; ak nie, skoc do slucky
F9A4 IN   4C      ; precitaj prijaty byte
F9A6 CPI  AA      ; je to prezentacny byte AA ?
F9A8 STC          ; nastav CY ako priznak chyby
F9A9 RNZ          ; nie je to prezentacny byte AA, vrat sa s CY
F9AA CALL F981    ; povol prerusenie pre vystup
F9AD MVI  A,AA    ; posli ako odpoved prezentacny byte AA
F9AF JMP  F958


;------------------------------------------------------------------------------
; Kratke zdrzanie
; vstup: -
; vystup: kratke zdrzanie asi 90 us
; meni: AF
;------------------------------------------------------------------------------
F9B2 MVI  A,14
F9B4 DCR  A
F9B5 JNZ  F9B4
F9B8 RET


;------------------------------------------------------------------------------
; Zisti, ci disk prijal prezentacny byte AA.
; vstup: -
; vystup: CY = 1 - chyba
;         CY = 0 - OK
; meni: AF, BC
;------------------------------------------------------------------------------
F9B9 CALL F978    ; povol prerusenie pre vstup
F9BC CALL F9B2    ; male zdrzanie asi 90 us
F9BF IN   4E
F9C1 ANI  08      ; je platny byte na vstupe?
F9C3 JZ   F981    ; ak nie, prezentacny byte bol prijaty
; a vrat sa cez povolenie prerusenia pre vystup
F9C6 CALL F978    ; povol prerusenie pre vstup
F9C9 IN   4C      ; precitaj byte zo vstupu
F9CB CPI  AA      ; ak je to prezentacny byte AA
F9CD JZ   F9E2    ; skoc poslat prezentacny byte AA na vystup
F9D0 LXI  B,1388  ; timeout asi 125 ms
F9D3 DCX  B       ; zniz pocitadlo timeoutu
F9D4 MOV  A,B     ; a otestuj ho na nulu
F9D5 ORA  C
F9D6 STC          ; nastav CY ako priznak timeoutu
F9D7 RZ           ; vrat sa, ak doslo k timeoutu
F9D8 IN   4E      ; prisiel novy byte?
F9DA ANI  08
F9DC JZ   F9D3    ; nie, skoc do slucky
F9DF JMP  F9C9    ; ano, skoc precitat byte zo vstupu

F9E2 CALL F9B2    ; male zdrzanie asi 90 us
F9E5 CALL F981    ; povol prerusenie pre vystup
F9E8 MVI  A,AA    ; posli na vystup prezentacny byte AA
F9EA OUT  4C
F9EC LXI  B,1388  ; timeout asi 125 ms
F9EF DCX  B       ; zniz pocitadlo timeoutu
F9F0 MOV  A,B     ; otestuj ho na nulu
F9F1 ORA  C
F9F2 JZ   F9C6    ; ak nebol prezentacny byte prijaty, skoc otestovat vstup
F9F5 IN   4E      ; bol prijaty prezentacny byte?
F9F7 ANI  08
F9F9 JZ   F9EF    ; ak nie, skoc do slucky
F9FC RET          ; bol prijaty prezentacny byte, vrat sa s vynulovanym CY


;------------------------------------------------------------------------------
; Posle byte v B na vystup a pocka na ACK (33) a vysledok povelu.
; vstup: B = posielany byte
; vystup: CY = 1 - chyba
;         CY = 0 - OK
; meni: AF, BC, DE
;------------------------------------------------------------------------------
F9FD MOV  A,B     ; posielany byte do A
F9FE CALL F958    ; posli na vystup
FA01 RC           ; vrat sa, ak doslo ku chybe
FA02 CALL F978    ; povol prerusenie na vstupe
FA05 CALL F943    ; precitaj byte zo vstupu
FA08 RC           ; vrat sa, ak doslo ku chybe
FA09 CPI  33      ; bolo prijate byte ACK?
FA0B STC          ; nastav chybovy priznak
FA0C RNZ          ; vrat sa s chybou, ak to nebolo ACK
FA0D LXI  D,0BB8  ; timeout asi 15,5 sec
FA10 CALL F961    ; je platny byte na vstupe?
FA13 JNC  F943    ; ak ano, skoc ho precitat
FA16 DCX  D       ; zniz pocitadlo timeoutu
FA17 MOV  A,D     ; a testuj ho na nulu
FA18 ORA  E
FA19 JNZ  FA10    ; ak nie je nulovy, skoc do slucky
FA1C STC          ; nastav priznak chyby
FA1D RET


;------------------------------------------------------------------------------
; Odosle CRC a caka na potvrdenie
; vstup: B = posielane CRC
; vystup: CY = 1 - chyba
;         CY = 0 - OK
; meni: AF, BC, DE
;------------------------------------------------------------------------------
FA1E CALL F9FD    ; posli byte v B a cakaj ACK (33) a vysledok pocelu (00)
FA21 RC           ; vrat sa, ak doslo ku chybe
FA22 ORA  A       ; ak bol druhy byte skutocne 00, je to OK
FA23 RZ
FA24 STC          ; nastav priznak chyby
FA25 RET


;------------------------------------------------------------------------------
; Posle do PMD 32 cislo sektoru a stopy
; vstup: cislo sektoru a mechaniky na adrese BFFC a cislo stopy na adrese BFFE
; vystup: CY = 1 - chyba
;         CY = 0 - OK, B = A = modifikovane CRC
; meni: AF, B
;------------------------------------------------------------------------------
FA26 LDA  BFFC    ; cislo sektoru a mechaniky
FA29 CALL F958    ; posli byte
FA2C RC           ; vrat sa pri chybe
FA2D LDA  BFFE    ; cislo stopy
FA30 JMP  F958    ; vrat sa cez odoslanie bytu


;------------------------------------------------------------------------------
; Precita byte zo vstupu, predstavujuci CRC
; vstup: -
; vystup: CY = 1 - chyba CRC alebo timeout
;         CY = 0 - CRC OK
; meni: AF, BC
;------------------------------------------------------------------------------
FA33 CALL F943    ; precitaj CRC zo vstupu
FA36 RC           ; vrat sa, ak doslo ku chybe
FA37 MOV  A,B     ; skontroluj CRC
FA38 ORA  A
FA39 RZ           ; CRC je OK
FA3A STC          ; nastav priznak chyby
FA3B RET


;------------------------------------------------------------------------------
; Povel "*" - rezim FAST
;        |-------------------------|------|-------------------------|
;        |        PMD 85-3         | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "*"                     | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK ERR                 |
;        |-------------------------|------|-------------------------|
; ACK = 33h; v pripade nejakej chyby je to NAK = 99h
; ERR = 00h, ak prebehol povel v poriadku, ina hodnota znamena chybu
; vstup: -
; vystup: CY = 1 - chyba timeout
;         CY = 0 - OK
; meni: AF, BC, DE
;------------------------------------------------------------------------------
FA3C MVI  A,2A    ; "*" - rezim FAST
FA3E CALL F94E    ; posli povel do disketovej jednotky
FA41 RC           ; vrat sa, ak doslo ku chybe
FA42 JMP  FA1E    ; posli CRC a pockaj na potvrdenie


;------------------------------------------------------------------------------
; Povel "@" - rezim SLOW
;        |-------------------------|------|-------------------------|
;        |        PMD 85-3         | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "@"                     | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK ERR                 |
;        |-------------------------|------|-------------------------|
; ACK = 33h; v pripade nejakej chyby je to NAK = 99h
; ERR = 00h, ak prebehol povel v poriadku, ina hodnota znamena chybu
; vstup: -
; vystup: CY = 1 - chyba timeout
;         CY = 0 - OK
; meni: AF, BC, DE
;------------------------------------------------------------------------------
FA45 MVI  A,40    ; "@" - rezim SLOW
FA47 CALL F94E    ; posli povel do disketovej jednotky
FA4A RC           ; vrat sa, ak doslo ku chybe
FA4B JMP  FA1E    ; posli CRC a pockaj na potvrdenie


;------------------------------------------------------------------------------
; Do PMD 32 posle povel, cislo sektoru a mechaniky, cislo stopy, vynuluje CRC
; a pripravi pocitadlo bytov (128) a adresu pamati.
; Rutina je volana pre povel "Q" - precitanie sektora.
; vstup: cislo sektora a mechaniky na BFFC, cislo stopy na BFFE
;        cielova adresa na BFF6
;        A = povel
; vystup: CY = 1 - pri nejakej chybe
;         CY = 0 - odoslany povel, sektor a stopa do PMD 32
;                  HL = adresa pamate z BFF6, B = 0, D = 128
; meni: vsetky
;------------------------------------------------------------------------------
FA4E CALL F94E    ; posli povel do disketovej jednotky
FA51 RC           ; vrat sa, ak doslo ku chybe
FA52 CALL FA26    ; posli stopu a sektor
FA55 RC           ; vrat sa, ak doslo ku chybe
FA56 CALL FA1E    ; posli CRC a pockaj na potvrdenie
FA59 MVI  B,00    ; vynuluj CRC
FA5B MVI  D,80    ; nastav pocitadlo
FA5D LHLD BFF6    ; do HL adresa pamati
FA60 RET


;------------------------------------------------------------------------------
; Zisti, ci je pripojena PMD 32, posle povel "B" a precita BOOT.
;        |-------------------------|------|-------------------------|
;        |        PMD 85-3         | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "B"                     | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK ERR                 |
;        |                         | <--- | 128 bytov dat           |
;        |                         | <--- | CRC                     |
;        |-------------------------|------|-------------------------|
; ACK = 33h; v pripade nejakej chyby je to NAK = 99h
; ERR = 00h, ak prebehol povel v poriadku, ina hodnota znamena chybu
; vstup: cielova adresa na BFF6
; vystup: CY = 1 - chyba timeout, CRC
;         CY = 0 - OK
; meni: vsetky
;------------------------------------------------------------------------------
FA61 CALL F98A    ; zisti, ci je pripojena disketova jednotka
FA64 RC           ; vrat sa, ak nie
FA65 MVI  A,42    ; "B" - precitaj BOOT
FA67 CALL F94E    ; odosli na vystup
FA6A RC           ; vrat sa, ak doslo ku chybe
FA6B CALL FA1E    ; odosli CRC a pockaj na potvrdenie
FA6E MVI  B,00    ; vynuluj CRC
FA70 MVI  D,80    ; pocitadlo 128 bytov
FA72 LHLD BFF6    ; vyber adresu do HL
FA75 XRA  A       ; vynuluj stopu a sektor
FA76 STA  BFFC
FA79 STA  BFFE
FA7C JMP  FA85    ; skoc precitat BOOT


;------------------------------------------------------------------------------
; Povel "Q" - precitanie sektora z disku.
;        |-------------------------|------|-------------------------|
;        |        PMD 85-3         | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "Q"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK ERR                 |
;        |                         | <--- | 128 bytov dat           |
;        |                         | <--- | CRC                     |
;        |-------------------------|------|-------------------------|
; ACK = 33h; v pripade nejakej chyby je to NAK = 99h
; ERR = 00h, ak prebehol povel v poriadku, ina hodnota znamena chybu
; vstup: cislo sektora a mechaniky na BFFC, cislo stopy na BFFE
;        cielova adresa na BFF6
; vystup: CY = 1 - chyba timeout, CRC
;         CY = 0 - OK
; meni: vsetky
;------------------------------------------------------------------------------
FA7F MVI  A,51    ; "Q" - precitaj sektor
FA81 CALL FA4E    ; posli povel, nastav stopu, sektor, HL, D a B (CRC)
FA84 RC           ; vrat sa, ak doslo ku chybe
FA85 IN   4E      ; je na vstupe platny byte?
FA87 ANI  08
FA89 JZ   FA85    ; ak nie, cakaj
FA8C IN   4C      ; precitaj byte
FA8E MOV  M,A     ; uloz do pamati
FA8F XRA  B       ; modifikuj CRC
FA90 MOV  B,A     ; a vrat ho do B
FA91 INX  H       ; posun ukazovatel
FA92 DCR  D       ; zniz pocitadlo prijimanych bytov
FA93 JNZ  FA85    ; ak neboli prijate vsetky, vrat sa do slucky
FA96 JMP  FA33    ; skoc precitat CRC


;------------------------------------------------------------------------------
; Povel "T" - zapise sektor na disk.
;        |-------------------------|------|-------------------------|
;        |        PMD 85-3         | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "T"                     | ---> |                         |
;        | cislo sektora           | ---> |                         |
;        | cislo stopy             | ---> |                         |
;        | 128 bytov dat           | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK ERR                 |
;        |-------------------------|------|-------------------------|
; ACK = 33h; v pripade nejakej chyby je to NAK = 99h
; ERR = 00h, ak prebehol povel v poriadku, ina hodnota znamena chybu
; vstup: cislo sektora a mechaniky na BFFC, cislo stopy na BFFE
;        zdrojova adresa na BFF6
; vystup: CY = 1 - chyba timeout, CRC
;         CY = 0 - OK
; meni: vsetky
;------------------------------------------------------------------------------
FA99 MVI  A,54    ; "T" - zapis sektor
FA9B CALL F94E    ; posli povel
FA9E RC           ; vrat sa, ak doslo ku chybe
FA9F CALL FA26    ; nastav hlavu na pozadobanu stopu a sektor
FAA2 RC           ; vrat sa, ak doslo ku chybe
FAA3 LHLD BFF6    ; adresa pamati do HL
FAA6 MVI  C,80    ; pocitadlo 128 bytov
FAA8 IN   4E      ; zisti, ci je volny vystup
FAAA ANI  08
FAAC JZ   FAA8    ; ak nie, cakaj
FAAF MOV  A,M     ; precitaj byte z pamati
FAB0 OUT  4C      ; posli na vystup
FAB2 XRA  B       ; modifikuj CRC
FAB3 MOV  B,A     ; a uloz naspat do B
FAB4 INX  H       ; posun adresu v HL
FAB5 DCR  C       ; zniz pocitadlo bytov
FAB6 JNZ  FAA8    ; opakuj pre vsetkych 128 bytov
FAB9 JMP  FA1E    ; odosli CRC a cakaj na potvrdenie


;------------------------------------------------------------------------------
; Povel "I" - volba aktivnej mechaniky a posun hlavy na stopu 0
;        |-------------------------|------|-------------------------|
;        |        PMD 85-3         | smer |        PMD 32           |
;        |-------------------------|------|-------------------------|
;        | "I"                     | ---> |                         |
;        | cislo mechaniky         | ---> |                         |
;        | CRC                     | ---> |                         |
;        |                         | <--- | ACK ERR                 |
;        |-------------------------|------|-------------------------|
; ACK = 33h; v pripade nejakej chyby je to NAK = 99h
; ERR = 00h, ak prebehol povel v poriadku, ina hodnota znamena chybu
; vstup: cislo mechaniky na BFF8
; vystup: CY = 1 - chyba timeout, CRC
;         CY = 0 - OK
; meni: AF, BC, DE
;------------------------------------------------------------------------------
FABC MVI  A,49    ; "I"
FABE CALL F94E    ; posli povel
FAC1 RC           ; vrat sa, ak doslo ku chybe
FAC2 LDA  BFF8    ; do A cislo mechaniky
FAC5 CALL F958    ; posli na vystup
FAC8 RC           ; vrat sa, ak doslo ku chybe
FAC9 JMP  FA1E    ; odosli CRC a cakaj na potvrdenie


;------------------------------------------------------------------------------
; Pripravi novu cielovu/zdrojovu adresu na BFF6
;   (BFF6) <= HL <= BC
; Je volane ako vstupny bod z adresy FF64.
; vstup: BC = adresa pamati
; vystup: nova adresa na BFF6
; meni: HL
;------------------------------------------------------------------------------
FACC MOV  H,B
FACD MOV  L,C
FACE SHLD BFF6
FAD1 RET


;------------------------------------------------------------------------------
; Pripravi cislo storpy na BFFE
;   (BFFE) <= A <= C
; Je volane ako vstupny bod z adresy FF61.
; vstup: C = cislo stopy 0 az n
; vystup: nove cislo stopy na BFFE
; meni: A
;------------------------------------------------------------------------------
FAD2 MOV  A,C
FAD3 STA  BFFE
FAD6 RET


;------------------------------------------------------------------------------
; Modifikuje cislo sektora na adrese BFFC a ponecha cislo mechaniky.
;   (BFFC) <= (((BFFC) AND C0) OR (C - 1))
; Je volane ako vstupny bod z adresy FF5E.
; vstup: C = cislo sektora 1 az 36
; vystup: nove cislo sektora na BFFC
; meni: AF
;------------------------------------------------------------------------------
FAD7 LDA  BFFC    ; vyber aktualne cislo stopy
FADA ANI  C0      ; odpamataj bity oznacujuce vybranu mechaniku
FADC DCR  C       ; uprav cislo sektora v C na rozsah 0 az n
FADD ORA  C       ; pridaj cislo sektora
FADE STA  BFFC    ; uloz nove cislo sektora
FAE1 INR  C       ; obnov C na povodnu hodnotu
FAE2 RET


;------------------------------------------------------------------------------
; Ulozi C do HL
;   HL <= C
; vstup: C = hodnota
; vystup: HL = C
; meni: HL
;------------------------------------------------------------------------------
FAE3 MOV  L,C
FAE4 MVI  H,00
FAE6 RET


;------------------------------------------------------------------------------
; Precita sektor z disku.
; Je volane ako vstupny bod z adresy FF58.
; vstup: cislo sektora a mechaniky na BFFC, cislo stopy na BFFE
;        cielova adresa na BFF6
; vystup: NZ, A = 1 - chyba timeout, CRC
;         Z, A = 0 - OK
; meni: vsetky
;------------------------------------------------------------------------------
FAE7 CALL FA7F    ; povel "Q"
FAEA MVI  A,00
FAEC ACI  00
FAEE RET


;------------------------------------------------------------------------------
; Zapise sektor na disk.
; Je volane ako vstupny bod z adresy FF5B.
; vstup: cislo sektora a mechaniky na BFFC, cislo stopy na BFFE
;        zdrojova adresa na BFF6
; vystup: NZ, A = 1 - chyba timeout, CRC
;         Z, A = 0 - OK
; meni: vsetky
;------------------------------------------------------------------------------
FAEF CALL FA99    ; povel "T"
FAF2 MVI  A,00
FAF4 ACI  00
FAF6 RET


;------------------------------------------------------------------------------
; Volba aktivnej mechaniky a posun hlavy na stopu 0.
; Je volane ako vstupny bod z adresy FF4C.
; vstup: cislo mechaniky na BFF8
; vystup: vynulovane cislo stopy na BFFE
;         NZ, A = 1 - chyba timeout, CRC
;         Z, A = 0 - OK
; meni: AF, BC, DE
;------------------------------------------------------------------------------
FAF7 CALL FABC    ; povel "I"
FAFA MVI  A,00
FAFC STA  BFFE
FAFF ACI  00
FB01 RET


;------------------------------------------------------------------------------
; Storbovany vystup znaku na GPIO-1 (kanal 4/1, konektor K4, port B 8255)
; Je volane ako vstupny bod z adresy FF67.
; vstup: C = posielany znak
; vystup: -
; meni: AF
;------------------------------------------------------------------------------
FB02 IN   4E
FB04 RAR
FB05 JNC  FB02
FB08 MOV  A,C
FB09 OUT  4D
FB0B RET


;------------------------------------------------------------------------------
; Ulozi do akumulatora FFh
; Je volane ako vstupny bod z adresy FF6A.
; vstup: -
; vystup: A = FF
; meni: A
;------------------------------------------------------------------------------
FB0C MVI  A,FF
FB0E RET


;------------------------------------------------------------------------------
; Precita sekvencne niekolko sektorov za sebou od 1. logickeho sektora
; (od 1. sektora za BOOTom).
; Pouziva sa pre nacitanie systemu pri studenom a teplom starte.
; Je volane ako vstupny bod z adresy FF6D.
; vstup: HL = adresa kam precitat sektory, C  - pocet sektorov
; vystup:
; meni: vsetky
;------------------------------------------------------------------------------
FB0F XCHG         ; odloz si HL do DE
FB10 LHLD BFF6    ; povodnu adresu z BFF6
FB13 PUSH H       ; na zasobnik
FB14 XCHG         ; vrat adresu z DE do HL
FB15 SHLD BFF6    ; a uloz na BFF6
FB18 XRA  A       ; 0. stopa
FB19 STA  BFFE
FB1C INR  A       ; 1. sektor
FB1D STA  BFFC
FB20 MVI  B,05    ; 4 pokusy
FB22 DCR  B       ; zniz pocitadlo pokusov
FB23 JNZ  FB35    ; a skoc, ak este neboli vsetky
FB26 MVI  A,AD    ; inak inicializuj GPIO 8255
FB28 OUT  4F      ; SK A: MOD 1, A-OUT, C5,C4-IN,  SK B: MOD 1, B-OUT
FB2A MVI  A,0D    ; 1->C6 povolenie prerusovacieho signalu INTR pre vystup A
FB2C OUT  4F
FB2E MVI  A,05    ; 1->C2 povolenie prerusovacieho signalu INTR pre vystup B
FB30 OUT  4F
FB32 JMP  E02D    ; skoc do monitora

FB35 PUSH B       ; odloz pocitadlo pokusov
FB36 CALL FA7F    ; precitaj sektor z disku
FB39 POP  B       ; obnov pocitadlo pokusov
FB3A JC   FB22    ; ak doslo ku chybe, skus to este raz
FB3D LHLD BFF6    ; vyber adresu kam sa uklada sektor
FB40 LXI  D,0080  ; dlzka sektora 128 bytov
FB43 DAD  D       ; pripocitaj dlzku sektora
FB44 SHLD BFF6    ; uloz novu adresu
FB47 LDA  BFFC    ; zvys cislo sektora
FB4A INR  A
FB4B STA  BFFC
FB4E CPI  1A      ; dosiahol si posledny sektor v stope?
FB50 JNZ  FB5B    ; nie, skoc
FB53 XRA  A       ; vynuluj cislo sektora
FB54 STA  BFFC
FB57 LXI  H,BFFE  ; adresa cisla stopy
FB5A INR  M       ; zvys cislo stopy
FB5B DCR  C       ; zniz pocet citanych sektorov
FB5C JNZ  FB20    ; a ak neboli precitane vsetky, vrat sa do slucky
FB5F POP  H       ; obnov povodnu adresu zo zasobnika
FB60 SHLD BFF6    ; a uloz
FB63 RET


;------------------------------------------------------------------------------
; Cast konzolovych rutin BIOS-u
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; Rutina pre tlac na obrazovku. Je volana sluzbou CONOUT (4) BIOS-u.
; Spracovava aj riadiace znaky.
; vstup: C = tlaceny znak
; vystup: -
; meni: vsetky
;------------------------------------------------------------------------------
; Riadiace kody:
; ---------------
; 01 - ATTR0  - zobrazovanie farbou 0
; 02 - ATTR1  - zobrazovanie farbou 1
; 03 - ATTR2  - zobrazovanie farbou 2
; 04 - ATTR3  - zobrazovanie farbou 3
; 08 - LEFT   - posun kurzora dolava
; 09 - HTAB   - posun kurzora o 8 znakov doprava
; 0A - DOWN   - kurzor dole (LF)
; 0D - CR     - kurzor na zaciatok riadku
; 0E - RIGHT  - posun kurzora doprava
; 0F - UP     - kurzor hore
; 1A - NORM   - normalne zobrazovanie (neinverzne)
; 1B - ESC    - nastavenie pozicie kurzora, potom - riadok + 32, stlpec + 32
; 1C - CLRL   - zrusenie aktualneho riadku (nasledujuce sa odroluju hore)
; 1D - INV    - inverzne zobrazovanie
; 1E - HOME   - kurzor do laveho horneho rohu obraovky
; 1F - CLS    - zmazanie celej obrazovky + HOME
;------------------------------------------------------------------------------
FCE0 LDA  C33C    ; spracovava sa ESC sekvencia ?
FCE3 ANA  A
FCE4 JNZ  FE2A    ; ano, skoc dopredu
FCE7 MOV  A,C     ; znak do A
FCE8 CPI  20      ; je to riadiaci znak ?
FCEA JNC  E500    ; nie, skoc vytlacit znak
FCED LHLD C03E    ; adresa kurzoru do HL
FCF0 CPI  1E      ; HOME - kurzor do laveho horneho rohu obraovky
FCF2 JNZ  FD00
FCF5 LXI  D,C040  ; DE = adresa 1. mikroriadku obrazovky
FCF8 LHLD C0FC    ; HL = sirka riadku (offset)
FCFB DAD  D       ; adresa kurzoru 0. riadku
FCFC SHLD C03E    ; uloz adresu kurzoru
FCFF RET

FD00 CPI  1F      ; CLS - zmazanie obrazovky a kurzor dolaveho horneho rohu
FD02 JNZ  FD0D
FD05 MVI  A,1C
FD07 CALL E500    ; zmaz obrazovku
FD0A JMP  FCF5    ; a nastav kurzor do laveho horneho rohu

FD0D CPI  0A      ; DOWN - kurzor dole (LF)
FD0F JNZ  FD22
FD12 MOV  A,L     ; odpamataj si poziciu v riadku
FD13 ANI  3F 
FD15 PUSH PSW
FD16 MVI  A,0D    ; prechod na zaciatok noveho riadku
FD18 CALL E500
FD1B LXI  H,C03E  ; adresa adresy kurzoru
FD1E POP  PSW     ; obnov povodnu poziciu v riadku
FD1F ADD  M       ; a uprav poziciu kurzora
FD20 MOV  M,A
FD21 RET

FD22 CPI  09      ; TAB - posun kurzora o 8 pozicii doprava
FD24 JNZ  FD3C
FD27 MVI  A,08    ; posun poziciu kurzora o 8
FD29 ADD  L
FD2A ANI  3F
FD2C CPI  30      ; dostal si sa mimo obrazovku ?
FD2E MOV  E,A
FD2F JC   FD34    ; nie, skoc dalej
FD32 MVI  E,2F    ; ano, nastav sa na pravy okraj
FD34 MOV  A,L
FD35 ANI  C0      ; uprav poziciu kurzora
FD37 ADD  E
FD38 MOV  L,A
FD39 JMP  FCFC    ; uloz novu poziciu kurzora

FD3C CPI  0E      ; RIGHT - posun kurzora doprava
FD3E JNZ  FD48
FD41 MOV  A,L
FD42 ANI  3F      ; odmaskuj poziciu v riadku
FD44 INR  A       ; o 1 doprava
FD45 JMP  FD2C    ; a spracuj

FD48 CPI  08      ; LEFT - posun kurzora dolava
FD4A JNZ  FD55
FD4D MOV  A,L
FD4E ANI  3F      ; ak si na zaciatku riadku,
FD50 RZ           ; vrat sa
FD51 DCR  A       ; o 1 dolava
FD52 JMP  FD2C    ; a spracuj

FD55 CPI  0F      ; UP - kurzor hore
FD57 JNZ  FD6C
FD5A PUSH H
FD5B XCHG         ; DE = adresa kurzora
FD5C LHLD C0FC    ; HL = sirka riadku (offset)
FD5F MOV  A,E     ; urob odpocet
FD60 SUB  L
FD61 MOV  L,A
FD62 MOV  A,D
FD63 SBB  H
FD64 MOV  H,A
FD65 CPI  C1      ; a ak si mimo obrazovky,
FD67 POP  D
FD68 RC           ; vrat sa bez zmeny
FD69 JMP  FCFC    ; uloz novu poziciu kurzora

FD6C CPI  1C      ; CLRL - zrusenie aktualbneho riadku
FD6E JNZ  FDBA
FD71 PUSH H       ; odpamataj adresu kurzora
FD72 LXI  D,FDAF  ; navratova adresa z rolovania obrazovky na zasobnik
FD75 PUSH D
FD76 MVI  A,C0    ; maska
FD78 ANA  L
FD79 MOV  L,A     ; kurzor na zaciatok riadku
FD7A XCHG         ; adresa kurzoru do DE - adresa nasledujuceho riadku
FD7B LXI  H,0000  ; odpamataj SP
FD7E DAD  SP
FD7F SHLD C13E
FD82 LHLD C0FC    ; HL = sirka riadku (offset)
FD85 MOV  A,E     ; vypocitaj adresu samotneho riadku
FD86 SUB  L
FD87 MOV  L,A
FD88 MOV  A,D
FD89 SBB  H
FD8A MOV  H,A
FD8B PUSH H       ; odpamataj adresu samoteneho riadku
FD8C DAD  H       ; vypocitaj pocet prenasanych mikroriadkov
FD8D DAD  H
FD8E LDA  C0F4    ; originalny pocet prenasanych mikroriadkov
FD91 STA  C33A    ; povodnu hodnotu si odpamataj
FD94 SUB  H       ; vypocitaj kolko mikroriadkov treba preniest
FD95 INR  A
FD96 STA  C0F4    ; uloz
FD99 POP  H       ; obnov adresu riadku
FD9A CPI  08      ; ak je to menej ako 8
FD9C JC   FDAB    ; zmaz iba tento riadok
FD9F LXI  B,FFC0  ; -40
FDA2 XCHG         ; HL = adresa kurzoru (nasledujuceho riadku)
FDA3 DAD  B       ; posun o 1 mikroriadok hore
FDA4 SPHL         ; SP = odkial
FDA5 XCHG
FDA6 DAD  B       ; HL = kam
FDA7 DCX  H
FDA8 JMP  BF00    ; odroluj a vrat sa na adresu FDAF cez obnovu SP a zmazanie
                  ; posledneho riadku

FDAB POP  D       ; zahod adresu riadku
FDAC CALL E0CA    ; zmaz iba aktualny riadok
FDAF LDA  C33A    ; obnov povodnu hodnotu presuvanych mikroriadkov
FDB2 STA  C0F4
FDB5 POP  H       ; obnov adresu kurzora
FDB6 SHLD C03E    ; a uloz ju
FDB9 RET

FDBA MOV  A,L     ; kurzor na zaciatok aktualneho riadku
FDBB ANI  C0
FDBD MOV  L,A
FDBE MOV  A,C     ; obnov ridiaci znak do A
FDBF CPI  0D      ; CR - kurzor na zaciatok riadku
FDC1 JZ   FCFC    ; uloz novu poziciu kurzoru
FDC4 CPI  1A      ; NORM - normalne zobrazenie (neinverzne)
FDC6 JNZ  FDD2
FDC9 LDA  C03A    ; kod farby
FDCC ANI  C0      ; ponechaj atributy a zrus masku
FDCE STA  C03A    ; uloz
FDD1 RET

FDD2 CPI  1D      ; INV - inverzne zobrazenie
FDD4 JNZ  FDE0
FDD7 LDA  C03A    ; kod farby
FDDA XRI  3F      ; invertuj masku
FDDC STA  C03A    ; uloz
FDDF RET

FDE0 CPI  01      ; ATTR0 - zobrazovanie farbou 0
FDE2 JNZ  FDEE
FDE5 LDA  C03A    ; kod farby
FDE8 ANI  3F      ; ponechaj masku a nastav atribut 00
FDEA STA  C03A
FDED RET

FDEE CPI  02      ; ATTR1 - zobrazovanie farbou 1
FDF0 JNZ  FDFE
FDF3 LDA  C03A    ; kod farby
FDF6 ANI  3F      ; ponechaj masku
FDF8 ORI  40      ; nastav atribut 01
FDFA STA  C03A    ; uloz
FDFD RET

FDFE CPI  03      ; ATTR2 - zobrazovanie farbou 2
FE00 JNZ  FE0E
FE03 LDA  C03A    ; kod farby
FE06 ANI  3F      ; ponechaj masku
FE08 ORI  80      ; nastav atribut 10
FE0A STA  C03A    ; uloz
FE0D RET

FE0E CPI  04      ; ATTR3 - zobrazovanie farbou 3
FE10 JNZ  FE1C
FE13 LDA  C03A    ; kod farby
FE16 ORI  C0      ; nastav atribut 11
FE18 STA  C03A    ; uloz
FE1B RET

FE1C CPI  1B      ; ESC - nastavenie pozicie kurzora
FE1E JNZ  FE27
FE21 MVI  A,01    ; nastav flag: spracuva sa ESC sekvencia
FE23 STA  C33C
FE26 RET

FE27 JMP  E500    ; neznamy riadiaci znak - nechaj spracovat povodnou rutinou

FE2A DCR  A       ; spracovanie ESC sekvencie
FE2B JNZ  FE3A
FE2E MVI  A,02    ; prisiel riadok - bude sa cakat este na stlpec
FE30 STA  C33C
FE33 MVI  A,E1    ; uprav rozsah na 1 az 28
FE35 ADD  C       ; C = cislo riadku + 32 => 32 az 59 pre 0. az 27. riadok
FE36 STA  C33D    ; uloz
FE39 RET

; C = cislo stlpca + 32 => 32 az 79 pre 0. az 47. stlpec
FE3A LHLD C0FC
FE3D LDA  C33D    ; A = cislo riadku
FE40 XCHG         ; DE = sirka riadku (offset)
FE41 LXI  H,C040  ; prvy textovy riadok
FE44 DAD  D       ; vypocitaj adresu pozadovaneho riadku
FE45 DCR  A
FE46 JNZ  FE44
FE49 MOV  A,C     ; stlpec
FE4A SUI  20      ; odpocitaj posun 32
FE4C CPI  30      ; konrola na opustenie obrazovky
FE4E JC   FE53
FE51 MVI  A,2F    ; nastav kurzor uplne doprava
FE53 ADD  L       ; uprav adresu kurora
FE54 MOV  L,A
FE55 XRA  A       ; ECS sekvencia je spracovana
FE56 STA  C33C
FE59 JMP  FCFC    ; vrat sa cez ulozenie novej adresy kurzoru


;------------------------------------------------------------------------------
; Radoby rutina pre tlac textu umiestneneho za volanim rutiny ukonceneho nulou.
; Kedze ale volana rutina pre tlac znaku na adrese FE98 meni vsetky registre
; a teda aj HL, strati sa po prvom volani tejto rutiny ukazovatel na text a
; zaroven aj navratova adresa, co povedie s najvacsou pravdepodobnostou k
; resetu.
; vstup: text za volanim funkcie, ukonceny nulou
; vystup: -
; meni: BC, DE, HL
;------------------------------------------------------------------------------
FE5C POP  H       ; adresa textu do HL
FE5D PUSH PSW     ; odpamataj AF
FE5E MOV  A,M     ; vyber znak
FE5F ANA  A       ; je to 0 ?
FE60 INX  H       ; posun na dalsi
FE61 JZ   FE6A    ; ak ano, skoc dopredu
FE64 CALL FE98    ; vytlac znak
FE67 JMP  FE5E    ; a znovu do slucky

FE6A POP  PSW     ; obnov AF
FE6B PCHL         ; navrat za tlaceny text


;------------------------------------------------------------------------------
; Rutina otestuje klavesnicu a jej kod ulozi na adresu C134.
; Rutina ma "chybicku krasy" a to v tom, ze transformovane kody klaves nie su
; ulozene na adresu C134 a sluzba CONIN volana bezprostredne po sluzbe CONST
; moze vratit nespravne kody klaves.
; vstup: -
; vystup: Z - nebola stlacena klavesa
;         NZ - klavesa stlacena - (C134) = A = kod stlacenej klavesy
; meni: vsetky
;------------------------------------------------------------------------------
FE6C CALL E9C0    ; otestuj klavesnicu bez cakania
FE6F CPI  87      ; kody klaves K0 az K11 (aj so SHIFTom) 88h az 9Fh
FE71 JC   FE7B
FE74 CPI  A0
FE76 JNC  FE7B
FE79 SUI  87      ; sa transformuju na CTRL-x kody 1 az 24
FE7B CPI  1D      ; klavesa DEL
FE7D JNZ  FE82
FE80 MVI  A,7F    ; sa vrati s hodnotou 7Fh
FE82 ANA  A
FE83 RET


;------------------------------------------------------------------------------
; Sluzba CONIN (3) BIOS-u. Necaka na stlacenie klavesy, ak bola predtym
; s uspechom volana sluzba CONST. V opacnom pripade pocka na stlacenie klavesy.
; vstup: -
; vystup: A = kod stlacenej klavesy
; meni: vsetky
;------------------------------------------------------------------------------
FE84 LDA  C134    ; zisti, ci bola predtym stlacena nejaka klavesa
FE87 ANA  A
FE88 JNZ  FE91    ; skoc dopredu, ak ano
FE8B CALL FE6C    ; otestuj klavesnicu
FE8E JZ   FE8B    ; cakaj na klavesu
FE91 PUSH PSW     ; odpamataj kod klavesy
FE92 XRA  A       ; zrus priznak stlacenej klavesy
FE93 STA  C134
FE96 POP  PSW     ; obnov kod klavesy
FE97 RET


;------------------------------------------------------------------------------
; Sluzba CONOUT (4) BIOS-u.
; Spracovava aj riadiace znaky a posuva kurzor.
; vstup: C = tlaceny znak
; vystup: -
; meni: vsetky
;------------------------------------------------------------------------------
FE98 PUSH B       ; odpamataj tlaceny znak
FE99 CALL E481    ; zmaz kurzor
FE9C POP  B       ; obnov tlaceny znak
FE9D MOV  A,C     ; tlaceny znak do A - zbytocna instrukcia
FE9E CALL FCE0    ; vytlac znak na obrazovku
FEA1 CALL E481    ; vykresli kurzor na novej pozicii (stacilo JP)
FEA4 RET          ; vrat sa


;------------------------------------------------------------------------------
; Sluzba CONST (2) BIOS-u. Zisti, ci bola stlacena klavesa
; ("Ukazkova rutina" ako sa nema programovat v assembleri...)
; vstup: -
; vystup: A = FF - bola stlacena klavesa, A = 00 - nebola stlacena klavesa
; meni: vsetky
;------------------------------------------------------------------------------
FEA5 LDA  C134    ; zisti, ci bola predtym stlacena nejaka klavesa
FEA8 ANA  A
FEA9 MVI  A,FF    ; A=FF - bola stlacena
FEAB JNZ  FEB6
FEAE CALL FE6C    ; otestuj klavesnicu bez cakania
FEB1 JZ   FEB6    ; nebola, A=00
FEB4 MVI  A,FF    ; A=FF - bola stlacena
FEB6 RET


;------------------------------------------------------------------------------
; Nasledujuce rutiny pracuju s obrazovkou a kvoli strankovaniu ROM s VideoRAM
; su premiestnene po starte systemu do RAM od adresy BF00.
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; Rutina pre rolovanie obrazovky
; vstup: SP = zdrojova adresa videoram
;        HL = cielova adresa videoram
;        na adrese C0F4 = pocet prenasanych mikroriadkov
; vystup: -
; meni: vsetky
;------------------------------------------------------------------------------
BF00 MVI  A,08    ; odstrankuj ROM
BF02 OUT  F7
BF04 LDA  C0F4    ; pocet prenasanych mikroriadkov
BF07 MVI  B,08    ; pocet cyklov 48/6
BF09 POP  D       ; dva byty z videoram
BF0A INX  H  
BF0B MOV  M,E     ; uloz na novu poziciu
BF0C INX  H
BF0D MOV  M,D
BF0E POP  D       ; dalsie dva byty
BF0F INX  H
BF10 MOV  M,E
BF11 INX  H
BF12 MOV  M,D
BF13 POP  D       ; tretia dvojica
BF14 INX  H
BF15 MOV  M,E
BF16 INX  H
BF17 MOV  M,D
BF18 DCR  B       ; celko 48 bytov na mikroriadok
BF19 JNZ  BF09
BF1C XCHG
BF1D LXI  H,0010
BF20 DAD  SP
BF21 SPHL         ; SP = dalsi zdrojovy mikroriadok
BF22 LXI  H,0010
BF25 DAD  D       ; HL = dalsi cielovy mikroriadok
BF26 DCR  A       ; znic pocitadlo mikroriadkov
BF27 JNZ  BF07
BF2A MVI  A,09    ; nastrankuj naspat ROM
BF2C OUT  F7
BF2E JMP  E0C6    ; a vrat sa cez obnovenie SP a zmazanie posledneho riadku


;------------------------------------------------------------------------------
; Rutina na rolovanie obsahu okien. (pouziva BASIC G 3.0)
; vstup: HL = zdrojova adresa vo videoram
;        DE = cielova adresa vo videoram
;        B = sirka okna v znakovych poziciach
;        C = vyska okna v bodoch
;        A = (64 - B) - offset pre prechod na dalsi mikroriadok
; vystup: -
; meni: vsetky
;------------------------------------------------------------------------------
BF31 PUSH PSW
BF32 MVI  A,08    ; odstrankuj ROM
BF34 OUT  F7
BF36 POP  PSW
BF37 PUSH B       ; odpamataj sirku a vysku okna
BF38 MOV  C,A     ; offset do A
BF39 MOV  A,M     ; byte v videoram
BF3A STAX D       ; skopiruj na ine miesto
BF3B INX  H       ; posun ukazovatel zdroja
BF3C INX  D       ; posun ukazovatel cielu
BF3D DCR  B       ; pocitadlo sirky okna
BF3E JNZ  BF39
BF41 DAD  B       ; posun zdroja na dalsi mikroriadok
BF42 XCHG
BF43 DAD  B       ; posun ciela na dalsi mikroriadok
BF44 XCHG
BF45 MOV  A,C     ; offset naspat do A
BF46 POP  B       ; obnov sirku a vysku
BF47 DCR  C       ; pocitadlo vysky okna
BF48 JNZ  BF37
BF4B PUSH PSW
BF4C MVI  A,09    ; nastrankuj naspat ROM
BF4E OUT  F7
BF50 POP  PSW
BF51 JMP  F0F9    ; zmaz spodny riadok okna po rolovani


;------------------------------------------------------------------------------
; Rutina vykreslenia bodu. (pouziva BASIC G 3.0)
; vstup: HL = adresa obrazovky, kde sa nachadza dany bod
;        B = maska bodu
; vystup: zobrazeny/zmazany bod
; meni: AF
;------------------------------------------------------------------------------
BF54 MOV  A,B     ; ak je maska nulova
BF55 ORA  A
BF56 RZ           ; vrat sa
BF57 MVI  A,08    ; odstrankuj ROM
BF59 OUT  F7
BF5B MOV  A,M     ; byte z obrazovky modifikuj podla nastaveni prikazu PEN x
BF5C CPI  3F      ; CPI 3F | ANI 3F
BF5E CPI  00      ; CPI 00 | ORI 00
BF60 NOP          ; NOP | CMA
BF61 XRA  B       ; XRA B | ORA B
BF62 NOP          ; NOP | CMA
BF63 MOV  M,A     ; uloz modifikovany byte
BF64 MVI  A,09    ; nastrankuj naspat ROM
BF66 OUT  F7
BF68 RET


;------------------------------------------------------------------------------
; Rutina zobrazenie/skrytie kurzoru
; vstup: HL = adresa obrazovky, kde je kurzor
; vystup: -
; meni: AF
;------------------------------------------------------------------------------
BF69 MVI  A,08    ; odstrankuj ROM
BF6B OUT  F7
BF6D MOV  A,M     ; byte z obrazovky
BF6E XRI  3E      ; zobraz/zmaz kurzor
BF70 JMP  BF63    ; akoc ulozit a prestrankovat naspat na ROM



;------------------------------------------------------------------------------
; Vstupne body do servisnych rutin CP/M BIOSu
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; SELDSK (9) - Select disc drive
; Volba aktivnej mechaniky a posun hlavy na stopu 0.
; vstup: cislo mechaniky na BFF8
; vystup: vynulovane cislo stopy na BFFE
;         A = 0 - OK
;         A = 1 - "neopravitelna" chyba
;------------------------------------------------------------------------------
FF4C JMP FAF7


;------------------------------------------------------------------------------
; CONIN (3) - console input
; test klavesnice, s upravou riadiacich kodov, caka na stlacenie, bez BEEP
; vstup: -
; vystup: A = kod stlacenej klavesy
;------------------------------------------------------------------------------
FF4F JMP FE84


;------------------------------------------------------------------------------
; CONOUT (4) - console output
; tlac znaku na obrazovku a posun kurzoru
; vstup: C = znak, ktory sa ma vytlacit na obrazovku
; vystup: tlac znaku
;------------------------------------------------------------------------------
FF52 JMP FE98


;------------------------------------------------------------------------------
; CONST (2) - console status
; otestuje klavesnicu
; vstup: -
; vystup: A = 0 - nestlacena klavesa
;         A = FF - stlacena klavesa
;------------------------------------------------------------------------------
FF55 JMP FEA5


;------------------------------------------------------------------------------
; READ (13) - Read a sector
; Citanie "aktualneho" sektora.
; vstup: -
; vystup: A = 0 - OK
;         A = 1 - "neopravitelna" chyba
;------------------------------------------------------------------------------
FF58 JMP FAE7


;------------------------------------------------------------------------------
; WRITE (14) - Write a sector
; Zapis do "aktualneho" sektora.
; vstup: -
; vystup: A = 0 - OK
;         A = 1 - "neopravitelna" chyba
;------------------------------------------------------------------------------
FF5B JMP FAEF


;------------------------------------------------------------------------------
; SETSEC (11) - Set sector number
; Modifikuje cislo sektora na adrese BFFC
; vstup: C = cislo sektora
; vystup: upravena hodnota na adrese BFFC
;------------------------------------------------------------------------------
FF5E JMP FAD7


;------------------------------------------------------------------------------
; SETTRK (10) - Set track number
; Modifikuje cislo stopy na adrese BFFE
; vstup: C = cislo stopy
; vystup: upravena hodnota na adrese BFFE
;------------------------------------------------------------------------------
FF61 JMP FAD2


;------------------------------------------------------------------------------
; SETDMA (12) - Set DMA address
; Modifikuje cielovu/zdrojovu adresu pamati pre zapis/citanie sektora
; vstup: BC = adresa pamati
; vystup: zmenena DMA adresa na adrese BFF6
;------------------------------------------------------------------------------
FF64 JMP FACC


;------------------------------------------------------------------------------
; LIST (5) - printer output
; Strobed OUT na GPIO-1. Caka, kym bude tlaciaren pripravena
; vstup: C = tlaceny znak
; vystup: znak odoslany na tlaciaren
;------------------------------------------------------------------------------
FF67 JMP FB02


;------------------------------------------------------------------------------
; LISTST (15) - list status - ci je pripravena tlaciaren
; vstup: -
; vystup: A = FF - vratcia stale, ze je tlaciaren pripravena
;------------------------------------------------------------------------------
FF6A JMP FB0C


;------------------------------------------------------------------------------
; Citanie skupiny sektorov systemovej oblasti diskety - CCP, BDOS, BIOS
; vstup: HL = adresa kam precitat sektory, C  - pocet sektorov
; vystup: -
;------------------------------------------------------------------------------
FF6D JMP FB0F
